{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "28e54144-8ef7-49a2-9bc3-e6f8ac7209de",
   "metadata": {},
   "source": [
    "# Kotlin: Tips Dataset"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "44dc9924-d27d-4c26-9e88-7fb89d19de63",
   "metadata": {},
   "source": [
    "The [*tips*](https://github.com/mwaskom/seaborn-data/blob/master/tips.csv) dataset is a data frame with 244 rows and 7 variables, which represents some tipping data where one waiter recorded information about each tip he received over a period of few months working in one restaurant."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "f98c7f9f-9810-4484-9a81-04d9a31a39c5",
   "metadata": {},
   "source": [
    "(short intro about the dataset)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "245b3823-0b5b-4222-9a73-56cb0411eb0e",
   "metadata": {},
   "source": [
    "For this example, we will import the following packages"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "c87f9ef0-5622-4af3-8fe7-bb88f9ba3556",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "            <div id=\"kotlin_out_0\"/>\n",
       "            <script type=\"text/javascript\">\n",
       "                            if(!window.kotlinQueues) {\n",
       "                window.kotlinQueues = {};\n",
       "            }\n",
       "            if(!window.kotlinQueues[\"DataFrame\"]) {\n",
       "                var resQueue = [];\n",
       "                window.kotlinQueues[\"DataFrame\"] = resQueue;\n",
       "                window[\"call_DataFrame\"] = function(f) {\n",
       "                    resQueue.push(f);\n",
       "                }\n",
       "            }\n",
       "            (function (){\n",
       "                var modifiers = [(function(script) {\n",
       "    script.src = \"https://cdn.jsdelivr.net/gh/Kotlin/dataframe@3db46ccccaa1291c0627307d64133317f545e6ae/core/src/main/resources/init.js\"\n",
       "    script.type = \"text/javascript\";\n",
       "})];\n",
       "                var e = document.getElementById(\"kotlin_out_0\");\n",
       "                modifiers.forEach(function (gen) {\n",
       "                    var script = document.createElement(\"script\");\n",
       "                    gen(script)\n",
       "                    script.addEventListener(\"load\", function() {\n",
       "                        window[\"call_DataFrame\"] = function(f) {f();};\n",
       "                        window.kotlinQueues[\"DataFrame\"].forEach(function(f) {f();});\n",
       "                        window.kotlinQueues[\"DataFrame\"] = [];\n",
       "                    }, false);\n",
       "                    script.addEventListener(\"error\", function() {\n",
       "                        window[\"call_DataFrame\"] = function(f) {};\n",
       "                        window.kotlinQueues[\"DataFrame\"] = [];\n",
       "                        var div = document.createElement(\"div\");\n",
       "                        div.style.color = 'darkred';\n",
       "                        div.textContent = 'Error loading resource DataFrame';\n",
       "                        document.getElementById(\"kotlin_out_0\").appendChild(div);\n",
       "                    }, false);\n",
       "                    \n",
       "                    e.appendChild(script);\n",
       "                });\n",
       "            })();\n",
       "            </script>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "                <style>\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "                </style>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "   <div id=\"vXWd8P\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"library\">\n",
       "       if(!window.letsPlotCallQueue) {\n",
       "           window.letsPlotCallQueue = [];\n",
       "       }; \n",
       "       window.letsPlotCall = function(f) {\n",
       "           window.letsPlotCallQueue.push(f);\n",
       "       };\n",
       "       (function() {\n",
       "           var script = document.createElement(\"script\");\n",
       "           script.type = \"text/javascript\";\n",
       "           script.src = \"https://cdn.jsdelivr.net/gh/JetBrains/lets-plot@v3.1.0/js-package/distr/lets-plot.min.js\";\n",
       "           script.onload = function() {\n",
       "               window.letsPlotCall = function(f) {f();};\n",
       "               window.letsPlotCallQueue.forEach(function(f) {f();});\n",
       "               window.letsPlotCallQueue = [];\n",
       "               \n",
       "               \n",
       "           };\n",
       "           script.onerror = function(event) {\n",
       "               window.letsPlotCall = function(f) {};\n",
       "               window.letsPlotCallQueue = [];\n",
       "               var div = document.createElement(\"div\");\n",
       "               div.style.color = 'darkred';\n",
       "               div.textContent = 'Error loading Lets-Plot JS';\n",
       "               document.getElementById(\"vXWd8P\").appendChild(div);\n",
       "           };\n",
       "           var e = document.getElementById(\"vXWd8P\");\n",
       "           e.appendChild(script);\n",
       "       })();\n",
       "   </script>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "%use multik\n",
    "%use dataframe\n",
    "%use lets-plot"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "3506e57a-2a55-469c-adf1-03890f3252b4",
   "metadata": {},
   "source": [
    "Let's load the \"Tips\" dataset, and show it's first 5 rows:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "35b24438-f80e-49a9-8f1b-1939dcfebb87",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":5,\"ncol\":7,\"columns\":[\"total_bill\",\"tip\",\"sex\",\"smoker\",\"day\",\"time\",\"size\"],\"kotlin_dataframe\":[{\"total_bill\":16.99,\"tip\":1.01,\"sex\":\"Female\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2},{\"total_bill\":10.34,\"tip\":1.66,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":3},{\"total_bill\":21.01,\"tip\":3.5,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":3},{\"total_bill\":23.68,\"tip\":3.31,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2},{\"total_bill\":24.59,\"tip\":3.61,\"sex\":\"Female\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":4}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648128\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 5, columnsCount = 7</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"total_bill: Double\\\">total_bill</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">16.99</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">10.34</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">21.01</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">23.68</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">24.59</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip: Double\\\">tip</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.01</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.66</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.50</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.31</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.61</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"sex: String\\\">sex</span>\", children: [], rightAlign: false, values: [\"Female\",\"Male\",\"Male\",\"Male\",\"Female\"] }, \n",
       "{ name: \"<span title=\\\"smoker: Boolean\\\">smoker</span>\", children: [], rightAlign: false, values: [\"false\",\"false\",\"false\",\"false\",\"false\"] }, \n",
       "{ name: \"<span title=\\\"day: String\\\">day</span>\", children: [], rightAlign: false, values: [\"Sun\",\"Sun\",\"Sun\",\"Sun\",\"Sun\"] }, \n",
       "{ name: \"<span title=\\\"time: String\\\">time</span>\", children: [], rightAlign: false, values: [\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\"] }, \n",
       "{ name: \"<span title=\\\"size: Int\\\">size</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\"] }, \n",
       "], id: 553648128, rootId: 553648128, totalRows: 5 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648128) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 2,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "val tips = DataFrame.readCSV(\"../resources/example-datasets/datasets/tips.csv\")\n",
    "tips.head()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "0be52e79-5c79-45e3-9964-95e58c3a1073",
   "metadata": {},
   "source": [
    "The dataset has 7 variables:\n",
    "- `total_bill` in dollars\n",
    "- `tip` in dollars\n",
    "- `sex` of the bill payer\n",
    "- `smokers` whether there were smokers in the party\n",
    "- `day` of the week\n",
    "- `time` time of day\n",
    "- `size`: people at the party"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "c07a0e22-ba92-49bb-ac70-d0c2eb1ba071",
   "metadata": {},
   "source": [
    "During the loading of the dataset, some values could have been mapped to a wrong datatype (e.g. `Date` can be loaded as `String` if not well formatted). \n",
    "\n",
    "With the `schema()` method it's possible to see how values have been parsed."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "7c764b46-f307-436c-b968-ee1f3f68d216",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "total_bill: Double\n",
       "tip: Double\n",
       "sex: String\n",
       "smoker: Boolean\n",
       "day: String\n",
       "time: String\n",
       "size: Int"
      ]
     },
     "execution_count": 3,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tips.schema()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "01f902f2-b22a-464e-bfa7-58cbf1547848",
   "metadata": {},
   "source": [
    "We can analyze some statistics of categorical data (`String` and `Boolean` columns):"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "881ac74d-25bc-4d2c-9ac6-568b04cac0b9",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":4,\"ncol\":10,\"columns\":[\"name\",\"type\",\"count\",\"unique\",\"nulls\",\"top\",\"freq\",\"min\",\"median\",\"max\"],\"kotlin_dataframe\":[{\"name\":\"sex\",\"type\":\"String\",\"count\":244,\"unique\":2,\"nulls\":0,\"top\":\"Male\",\"freq\":157,\"min\":\"Female\",\"median\":\"Male\",\"max\":\"Male\"},{\"name\":\"day\",\"type\":\"String\",\"count\":244,\"unique\":4,\"nulls\":0,\"top\":\"Sat\",\"freq\":87,\"min\":\"Fri\",\"median\":\"Sun\",\"max\":\"Thur\"},{\"name\":\"time\",\"type\":\"String\",\"count\":244,\"unique\":2,\"nulls\":0,\"top\":\"Dinner\",\"freq\":176,\"min\":\"Dinner\",\"median\":\"Dinner\",\"max\":\"Lunch\"},{\"name\":\"smoker\",\"type\":\"Boolean\",\"count\":244,\"unique\":2,\"nulls\":0,\"top\":\"false\",\"freq\":151,\"min\":\"false\",\"median\":\"false\",\"max\":\"true\"}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648129\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 4, columnsCount = 10</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"name: String\\\">name</span>\", children: [], rightAlign: false, values: [\"sex\",\"day\",\"time\",\"smoker\"] }, \n",
       "{ name: \"<span title=\\\"type: Any\\\">type</span>\", children: [], rightAlign: false, values: [\"String\",\"String\",\"String\",\"Boolean\"] }, \n",
       "{ name: \"<span title=\\\"count: Int\\\">count</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">244</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">244</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">244</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">244</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"unique: Int\\\">unique</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"nulls: Int\\\">nulls</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"top: Comparable\\\">top</span>\", children: [], rightAlign: false, values: [\"Male\",\"Sat\",\"Dinner\",\"false\"] }, \n",
       "{ name: \"<span title=\\\"freq: Int\\\">freq</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">157</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">87</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">176</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">151</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"min: Comparable\\\">min</span>\", children: [], rightAlign: false, values: [\"Female\",\"Fri\",\"Dinner\",\"false\"] }, \n",
       "{ name: \"<span title=\\\"median: Comparable\\\">median</span>\", children: [], rightAlign: false, values: [\"Male\",\"Sun\",\"Dinner\",\"false\"] }, \n",
       "{ name: \"<span title=\\\"max: Comparable\\\">max</span>\", children: [], rightAlign: false, values: [\"Male\",\"Thur\",\"Lunch\",\"true\"] }, \n",
       "], id: 553648129, rootId: 553648129, totalRows: 4 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648129) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 4,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tips.describe { colsOf<String>() and colsOf<Boolean>() }"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "a561b467-401a-4578-99f7-f02a46cc97dd",
   "metadata": {},
   "source": [
    "There are four categorical variables in the Tips dataset as seen above. For a better visualization of those data, we can make plots for visualizing for example the number of people for each day of the week."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "19c4695f-340e-4474-ab47-c96994a33b12",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"zoC8fj\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Tables served by Day and party Size\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"day\"\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..count..\",\n",
       "\"fill\":\"@as_discrete@size\"\n",
       "},\n",
       "\"stat\":\"count\",\n",
       "\"alpha\":0.8,\n",
       "\"position\":\"dodge\",\n",
       "\"geom\":\"bar\",\n",
       "\"data_meta\":{\n",
       "\"mapping_annotations\":[{\n",
       "\"aes\":\"fill\",\n",
       "\"annotation\":\"as_discrete\",\n",
       "\"parameters\":{\n",
       "\"label\":\"size\"\n",
       "}\n",
       "}]\n",
       "},\n",
       "\"data\":{\n",
       "\"..count..\":[2.0,1.0,1.0,39.0,53.0,48.0,16.0,15.0,18.0,4.0,1.0,18.0,13.0,5.0,1.0,1.0,3.0,1.0,3.0,1.0],\n",
       "\"@as_discrete@size\":[1.0,1.0,1.0,2.0,2.0,2.0,2.0,3.0,3.0,3.0,3.0,4.0,4.0,4.0,4.0,5.0,5.0,5.0,6.0,6.0],\n",
       "\"day\":[\"Sat\",\"Thur\",\"Fri\",\"Sun\",\"Sat\",\"Thur\",\"Fri\",\"Sun\",\"Sat\",\"Thur\",\"Fri\",\"Sun\",\"Sat\",\"Thur\",\"Fri\",\"Thur\",\"Sun\",\"Sat\",\"Thur\",\"Sun\"]\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"zoC8fj\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(tips.select { day and size }.sortBy(\"size\").toMap()) { x = \"day\" } +\n",
    "    geomBar(stat = Stat.count(), position=positionDodge(), alpha=0.8 ) { \n",
    "        y = \"..count..\" ; fill=asDiscrete(\"size\") \n",
    "    } +\n",
    "    ggtitle(\"Tables served by Day and party Size\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "387bf48c-d1b2-4e6b-8928-fbd0f90c54f5",
   "metadata": {},
   "source": [
    "- Fridays are the quietest days. Saturdays are the busiest followed by Sundays, meaning that there are more customers in the weekend.\n",
    "- The most common party size is by far *2*, and there are very a few lone diners."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "9a91470a-09cd-4c9d-9f71-99db8c38f512",
   "metadata": {},
   "source": [
    "````{margin}\n",
    "```{note}\n",
    "The `tooltips` option is used to customize the tooltip when you hover the mouse on the graph. Try it and see the result! (for more informations about how to customize the tooltip, visit the [tooltip guide](https://github.com/JetBrains/lets-plot-kotlin/blob/master/docs/tooltips.md)).\n",
    "```\n",
    "````"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "a617de14-7fa5-4f7a-b268-0bebb838112f",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"R2rxN5\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"kind\":\"ggbunch\",\n",
       "\"items\":[{\n",
       "\"x\":0,\n",
       "\"width\":400,\n",
       "\"y\":0,\n",
       "\"feature_spec\":{\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..prop..\",\n",
       "\"fill\":\"smoker\"\n",
       "},\n",
       "\"stat\":\"count\",\n",
       "\"alpha\":0.8,\n",
       "\"position\":\"fill\",\n",
       "\"geom\":\"bar\",\n",
       "\"tooltips\":{\n",
       "\"lines\":[\"perc. |@..prop..\"],\n",
       "\"variables\":[\"smoker\"],\n",
       "\"formats\":[{\n",
       "\"field\":\"..prop..\",\n",
       "\"format\":\".1%\"\n",
       "}]\n",
       "},\n",
       "\"data\":{\n",
       "\"smoker\":[false,false,false,false,true,true,true,true],\n",
       "\"..prop..\":[0.75,0.5172413793103449,0.7258064516129032,0.21052631578947367,0.4827586206896552,0.27419354838709675,0.7894736842105263,0.25],\n",
       "\"day\":[\"Sun\",\"Sat\",\"Thur\",\"Fri\",\"Sat\",\"Thur\",\"Fri\",\"Sun\"]\n",
       "}\n",
       "}],\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Percentage of smokers for each day\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"day\"\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[{\n",
       "\"aesthetic\":\"y\",\n",
       "\"format\":\".1%\"\n",
       "}]\n",
       "},\n",
       "\"height\":400\n",
       "},{\n",
       "\"x\":400,\n",
       "\"width\":400,\n",
       "\"y\":0,\n",
       "\"feature_spec\":{\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Percentage of bill payers' sex for each day\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"day\"\n",
       "},\n",
       "\"coord\":{\n",
       "\"name\":\"flip\",\n",
       "\"flip\":true\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[{\n",
       "\"aesthetic\":\"y\",\n",
       "\"format\":\".1%\"\n",
       "}],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..prop..\",\n",
       "\"fill\":\"sex\"\n",
       "},\n",
       "\"stat\":\"count\",\n",
       "\"alpha\":0.8,\n",
       "\"position\":\"fill\",\n",
       "\"geom\":\"bar\",\n",
       "\"tooltips\":{\n",
       "\"lines\":[\"perc. |@..prop..\"],\n",
       "\"variables\":[\"sex\"],\n",
       "\"formats\":[{\n",
       "\"field\":\"..prop..\",\n",
       "\"format\":\".1%\"\n",
       "}]\n",
       "},\n",
       "\"data\":{\n",
       "\"sex\":[\"Female\",\"Female\",\"Female\",\"Female\",\"Male\",\"Male\",\"Male\",\"Male\"],\n",
       "\"..prop..\":[0.23684210526315788,0.3218390804597701,0.5161290322580645,0.47368421052631576,0.7631578947368421,0.6781609195402298,0.4838709677419355,0.5263157894736842],\n",
       "\"day\":[\"Sun\",\"Sat\",\"Thur\",\"Fri\",\"Sun\",\"Sat\",\"Thur\",\"Fri\"]\n",
       "}\n",
       "}]\n",
       "},\n",
       "\"height\":400\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"R2rxN5\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "val p1 = ggplot(tips.select { day and smoker}.toMap()) { x = \"day\" } +\n",
    "    geomBar(\n",
    "        stat = Stat.count(),\n",
    "        position = positionFill(),\n",
    "        alpha = 0.8,\n",
    "        tooltips = layerTooltips(\"smoker\")\n",
    "            .format(\"..prop..\", \".1%\")\n",
    "            .line(\"perc. |@..prop..\")\n",
    "            \n",
    "    ) { y = \"..prop..\"; fill = \"smoker\"} +\n",
    "    scaleYContinuous(format=\".1%\") +\n",
    "    ggtitle(\"Percentage of smokers for each day\")\n",
    "\n",
    "val p2 = ggplot(tips.select { day and sex }.toMap()) { x = \"day\" } +\n",
    "    geomBar(\n",
    "        stat = Stat.count(),\n",
    "        position = positionFill(),\n",
    "        alpha = 0.8,\n",
    "        tooltips = layerTooltips(\"sex\")\n",
    "            .format(\"..prop..\", \".1%\")\n",
    "            .line(\"perc. |@..prop..\")\n",
    "        ) { y = \"..prop..\" ; fill = \"sex\" } +\n",
    "    coordFlip() +\n",
    "    scaleYContinuous(format=\".1%\") +\n",
    "    ggtitle(\"Percentage of bill payers' sex for each day\")\n",
    "\n",
    "GGBunch().addPlot(p1, 0, 0, 400, 400).addPlot(p2, 400, 0, 400, 400)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "d60c7b93-3970-4d84-b932-7b066ddb1c68",
   "metadata": {},
   "source": [
    "It's very easy now to notice that:\n",
    "- There are almost equal numbers of male and female that pay the bill in the weekday, but the number of male increases at the weekend.\n",
    "- The percentage of non smokers is most of the time major that the total percentage, but in the day with least people in the restaurant (Friday), most of them are smokers."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "65c3ef7e-99ae-4261-9e02-4e929c8b0714",
   "metadata": {},
   "source": [
    "Let's analyze now *quantitative variables*: `total_bill` and `tips`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "0444c91a-5cd2-4036-b6fd-ba5c59e55cdd",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"r8BFa0\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Total bill amounts frequencies\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"total_bill\"\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..density..\"\n",
       "},\n",
       "\"stat\":\"bin\",\n",
       "\"bins\":25.0,\n",
       "\"color\":\"black\",\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"histogram\",\n",
       "\"fill\":\"white\",\n",
       "\"data\":{\n",
       "\"..density..\":[0.0021461880266745417,0.0021461880266745417,0.010730940133372707,0.019315692240070875,0.049362324613514454,0.06438564080023625,0.049362324613514454,0.06867801685358534,0.049362324613514454,0.042923760533490826,0.0257542563200945,0.03433900842679267,0.017169504213396334,0.019315692240070875,0.015023316186721791,0.008584752106698167,0.015023316186721791,0.0021461880266745417,0.006438564080023625,0.006438564080023625,0.0021461880266745417,0.0042923760533490835,0.0,0.006438564080023625,0.0021461880266745417],\n",
       "\"total_bill\":[2.7415487999999995,4.7580864,6.774623999999999,8.791161599999999,10.807699199999998,12.824236799999998,14.8407744,16.857312,18.8738496,20.8903872,22.9069248,24.9234624,26.94,28.9565376,30.9730752,32.989612799999996,35.006150399999996,37.022687999999995,39.039225599999995,41.055763199999994,43.072300799999994,45.08883839999999,47.10537599999999,49.12191359999999,51.1384512]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "},\n",
       "\"stat\":\"density\",\n",
       "\"alpha\":0.2,\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"area\",\n",
       "\"fill\":\"orange\",\n",
       "\"data\":{\n",
       "\"..density..\":[0.002174949999265651,0.00228574231796987,0.00240212952484002,0.0025243784856906217,0.0026527617235622206,0.00278755672087752,0.0029290451628887965,0.0030775121251880705,0.00323324520848449,0.003396533624254756,0.003567667235237417,0.003746935555068402,0.0039346267116385225,0.004131026378994077,0.004336416682795233,0.004551075084496096,0.0047752732495124186,0.005009275904701877,0.005253339690497068,0.005507712013007043,0.005772629901341269,0.006048318875315307,0.006334991828572173,0.006632847932004242,0.006942071562189958,0.007262831259373818,0.007595278719320127,0.007939547823167853,0.008295753709205848,0.00866399189028361,0.009044337420370816,0.009436844113586383,0.009841543818833683,0.010258445753006716,0.010687535895571885,0.0111287764471817,0.011582105354841611,0.01204743590602506,0.01252465639401457,0.013013629856637069,0.013514193890452565,0.014026160542348228,0.014549316280378184,0.015083422045568715,0.015628213386278428,0.01618340067655412,0.016748669419757414,0.017323680638547428,0.017908071352087707,0.018501455141102293,0.019103422801129748,0.019713543084016925,0.020331363527352816,0.02095641137117139,0.021588194560845354,0.02222620283465902,0.02286990889408322,0.023518769654286672,0.024172227571909028,0.024829712046590256,0.02549064089221231,0.026154421873260333,0.026820454301158026,0.02748813068488564,0.02815683842964948,0.02882596157684594,0.02949488257806114,0.030162984095364095,0.03082965081970667,0.031494271298828716,0.032156239765694095,0.03281495795815799,0.033469836920284254,0.03412029877550699,0.03476577846166101,0.03540572541779235,0.036039605212613374,0.03666690110448138,0.0372871155228618,0.03789977146139062,0.038504413772874,0.039100610356858925,0.03968795323078507,0.040266059476174614,0.04083457205184971,0.04139316046677478,0.04194152130581198,0.04247937860245253,0.04300648405344015,0.04352261707113918,0.04402758467051815,0.04452122118871225,0.045003387836298134,0.04547397208065316,0.04593288686307453,0.04638006965269394,0.04681548134163084,0.047239104987273586,0.0476509444090478,0.048051022648514276,0.04843938030311659,0.04881607374535797,0.04918117324060398,0.0495347609780709,0.04987692903084193,0.0502077772619355,0.050527411194517036,0.05083593986526442,0.05113347368066187,0.05142012229657324,0.05142012229657324,0.05169599254183135,0.05196118640674007,0.0522157991173259,0.05245991731586745,0.052693617367679496,0.05291696381331649,0.053130007984299586,0.05333278679915143,0.053525321754963204,0.05370761812791976,0.05387966439419282,0.05404143188039678,0.05419287465040937,0.054333929632817544,0.054464516990591,0.05458454073184088,0.05469388955772975,0.05479243794079523,0.054880047424178735,0.05495656812954259,0.0550218404588702,0.05507569697289262,0.05511796442662736,0.05514846594048094,0.05516702328357975,0.055173459244501684,0.055167600063387175,0.055149277898558044,0.05511833330025272,0.05507461766393844,0.05501799563585865,0.054948347444045836,0.0548655711289395,0.05476958464901044,0.05466032783836796,0.054537764195201066,0.05440188248205105,0.054252698121291736,0.05409025437177553,0.053914623275342605,0.05372590636474952,0.05352423512750606,0.05330977122306934,0.05308270645379192,0.05284326249290454,0.052591690375600214,0.05232826976192647,0.05205330798265492,0.0517671388815444,0.0517671388815444,0.05147012146942093,0.05116263840723236,0.05084509433668692,0.050517914078231974,0.05018154071696383,0.049836433597585446,0.04948306624973184,0.04912192426489563,0.04875350314579184,0.04837830614834259,0.04799684213554768,0.04760962346136478,0.04721716390138388,0.0468199766455742,0.04641857236674129,0.04601345737659302,0.0456051318795105,0.04519408833228737,0.04478080991627311,0.0443657691265655,0.043949426481175,0.04353222935145221,0.04311461091356278,0.042696989219421236,0.0422797663842827,0.04186332788714555,0.04144804197925203,0.04103425919528802,0.0406223119613801,0.04021251429366513,0.0398051615810516,0.03940053044579987,0.03899887867569833,0.03860044522189463,0.03820545025682881,0.03781409528719713,0.03742656331742041,0.0370430190596854,0.03666360918724212,0.03628846262825978,0.03591769089814181,0.035551388468762865,0.03518963317359689,0.03483248664814112,0.034479994805395826,0.034132188346419505,0.03378908330614539,0.033450681634700066,0.03311697181442695,0.03278792951266826,0.03246351827011893,0.0321436902242353,0.03182838686676996,0.031517539834026324,0.031211071727894408,0.030908896965157705,0.030610922651964315,0.03031704947975515,0.030027172638347113,0.02974118274130129,0.02945896675818017,0.029180408947826374,0.0289053917863932,0.028633796883536643,0.02836550587994804,0.02810040131927621,0.027838367487460297,0.027579291212578606,0.027323062618509423,0.027323062618509423,0.027069575826002355,0.026818729595165482,0.026570427903882695,0.026324580457277043,0.026081103124024112,0.02583991829608041,0.025600955169216875,0.025364149942621445,0.02512944593674578,0.024896793629503016,0.024666150611863724,0.024437481464829824,0.02421075756067671,0.023985956792230572,0.023763063234774284,0.02354206674594235,0.023322962509660105,0.023105750530795652,0.022890435087716095,0.02267702415036572,0.022465528771807373,0.022255962461385365,0.022048340547777,0.021842679540198685,0.021638996495924653,0.021437308402060547,0.021237631579198286,0.021039981114165208,0.02084437032857732,0.020650810289320788,0.02045930936642659,0.020269872843079694,0.020082502581726645,0.01989719674942503,0.0197139496047264,0.019532751347511392,0.019353588032315832,0.019176441544808492,0.0190012896402194,0.01882810604168103,0.01865686059564576,0.018487519480791432,0.018320045466132293,0.018154398213424235,0.01799053461839932,0.017828409184891005,0.01766797442552629,0.017509181282366785,0.017351979560683437,0.017196318368949633,0.017042146558138085,0.016889413153505842,0.016738067772249805,0.016588061020708023,0.01643934486516654,0.016291872970802007,0.016145601003841086,0.01600048689264034,0.01585649104407653,0.015713576512378636,0.015571709118316773,0.015430857517481278,0.015290993217223402,0.015152090542676798,0.01501412655312404,0.014877080910800288,0.01474093570502776,0.014605675235335173,0.01447128575792433,0.014337755200492163,0.014205072850988258,0.014073229026376614,0.01394221472786808,0.013812021289388327,0.013682640026240369,0.01355406189100553,0.013426277143699731,0.013299275043061592,0.013173043565595488,0.013047569158628725,0.012922836533170766,0.01279882850178974,0.012675525866054086,0.012552907357334608,0.012430949633933068,0.01230962733661129,0.012188913203649999,0.012068778245585436,0.011949191978766312,0.011830122715861613,0.01171153791044408,0.011593404551793975,0.011475689605125058,0.01135836049154871,0.011241385601274785,0.011124734832815099,0.01100838015031989,0.01089229615065055,0.010776460631384668,0.010660855150669855,0.010545465569698096,0.010430282568567113,0.010315302126432679,0.010200525957135205,0.010085961891905679,0.009971624201313228,0.009857533849306268,0.009743718673009922,0.009630213482866226,0.009517060078726016,0.009404307178610148,0.00929201025803575,0.009180231299033677,0.009069038449249393,0.00895850559280032,0.008848711835841145,0.008739740911043105,0.008631680506406444,0.0085246215249772,0.008418657283111895,0.008313882655911544,0.008210393179311007,0.008108284119051271,0.008007649517364122,0.007908581228655757,0.007811167955776092,0.007715494298602421,0.007621639826643711,0.007529678187187503,0.007439676260165404,0.00735169337041168,0.0072657805673386885,0.007181979981262708,0.007100324264695756,0.007020836125886001,0.00694352796075762,0.00686840158818554,0.006795448092260755,0.006724647773874672,0.006655970212596977,0.0065893744384586395,0.006524809211900631,0.006462213408827055,0.006401516506428714,0.006342639164235145,0.006285493893727501,0.006229985808815192,0.006176013448558999,0.0061234696627245535,0.006072242550080695,0.006022216438826008,0.005973272898137879,0.005925291769595595,0.005878152207132748,0.005831733714223946,0.005785917167202521,0.005740585813935677,0.0056956262375429684,0.005650929275425852,0.005606390884569176,0.005561912944868988,0.0055174039931221545,0.005472779881268263,0.005427964353489497,0.005382889537833962,0.0053374963491182635,0.005291734800970038,0.0052455642259764,0.005198953403994578,0.005151880599743064,0.005104333511811107,0.00505630913618998,0.00500781354832839,0.004958861608537552,0.004909476596308945,0.004859689779752167,0.004809539926905643,0.004759072766113696,0.00470834040299739,0.004657400701770918,0.004606316638770722,0.004555155636071693,0.004503988882967188,0.004452890652890592,0.0044019376230622015,0.004351208203762148,0.00430078188366679,0.00425073859715041,0.004201158118856817,0.00415211949019656,0.004103700481735974,0.0040559770947263,0.004009023104285325,0.003962909646003475,0.003917704847011431,0.00387347350182968,0.003830276792632103,0.003788172052906622,0.00374721257289528,0.0037074474446529317,0.003668921444085247,0.003631674946919732,0.003595743875233024,0.00356115967090744,0.0035279492922227398,0.0034961352297053134,0.0034657355373571563,0.0034367638754681444,0.0034092295613746795,0.0033831376247605055,0.003358488864395913,0.0033352799035719126,0.0033135032418987916,0.0032931473015938772,0.0032741964668722132,0.0032566311155651515,0.00324042764261528,0.0032255584756203564,0.003211992083113125,0.0031996929767574445,0.003188621709103521,0.0031787348689667518,0.0031699850768666694,0.0031623209832769518,0.0031556872726875135,0.003150024676659906,0.003145269999163261,0.0031413561575073584,0.003138212242140481,0.003135763598453241,0.0031339319335269584,0.0031326354504900566,0.0031317890128028493,0.003131304340386191,0.0031310902390500397,0.003131052864172366,0.003131096019036385,0.003131121487664728,0.003131029401403566,0.0031307186379186205,0.0031300872506795645,0.003129032926440361,0.003127453467681206,0.00312524729647313,0.003122313975768137,0.003118554743714778,0.0031138730562584437,0.0031081751330139244,0.0031013705011996846,0.0030933725323025927,0.003084098966100572,0.0030734724167095604,0.0030614208554394346,0.0030478780654391367,0.003032784063380337,0.0030160854837670346,0.0029977359218592518,0.002977696231655689,0.0029559347758846854,0.0029324276254967343,0.0029071587067258284,0.002880119894381587,0.0028513110506400026,0.0028207400092077673,0.0027884225053345824,0.0027543820527301077,0.0027186497689992827,0.002681264151733428,0.0026422708078782686,0.0026017221394373504,0.002559676988955561,0.002516200248558313,0.0024713624365947,0.002425239246145753,0.002377911069810854,0.0023294625052772723,0.0022799818462102754,0.0022295605629774617,0.0021782927776430585,0.0021262747375407165,0.0020736042915603663,0.0020203803730718746,0.001966702493160015,0.0019126702475682196],\n",
       "\"total_bill\":[3.07,3.1634246575342466,3.256849315068493,3.3502739726027397,3.443698630136986,3.537123287671233,3.630547945205479,3.723972602739726,3.8173972602739727,3.910821917808219,4.004246575342465,4.097671232876712,4.191095890410959,4.284520547945205,4.3779452054794525,4.471369863013699,4.564794520547945,4.6582191780821915,4.751643835616438,4.845068493150685,4.938493150684931,5.031917808219179,5.125342465753425,5.218767123287671,5.312191780821918,5.405616438356164,5.499041095890411,5.592465753424658,5.685890410958905,5.779315068493151,5.8727397260273975,5.966164383561644,6.05958904109589,6.153013698630137,6.246438356164384,6.339863013698631,6.433287671232877,6.526712328767124,6.62013698630137,6.713561643835616,6.806986301369863,6.90041095890411,6.993835616438357,7.087260273972603,7.18068493150685,7.274109589041096,7.367534246575342,7.460958904109589,7.554383561643837,7.647808219178083,7.7412328767123295,7.834657534246576,7.928082191780822,8.021506849315069,8.114931506849315,8.208356164383561,8.30178082191781,8.395205479452056,8.488630136986302,8.582054794520548,8.675479452054795,8.768904109589041,8.862328767123287,8.955753424657535,9.049178082191782,9.142602739726028,9.236027397260274,9.32945205479452,9.422876712328767,9.516301369863013,9.609726027397262,9.703150684931508,9.796575342465754,9.89,9.983424657534247,10.076849315068493,10.17027397260274,10.263698630136988,10.357123287671234,10.45054794520548,10.543972602739727,10.637397260273973,10.73082191780822,10.824246575342466,10.917671232876714,11.01109589041096,11.104520547945206,11.197945205479453,11.2913698630137,11.384794520547947,11.478219178082194,11.57164383561644,11.665068493150686,11.758493150684933,11.851917808219179,11.945342465753425,12.038767123287673,12.13219178082192,12.225616438356166,12.319041095890412,12.412465753424659,12.505890410958905,12.599315068493151,12.692739726027398,12.786164383561646,12.879589041095892,12.973013698630139,13.066438356164385,13.159863013698631,13.253287671232878,13.253287671232878,13.346712328767124,13.440136986301372,13.533561643835618,13.626986301369865,13.720410958904111,13.813835616438357,13.907260273972604,14.00068493150685,14.094109589041098,14.187534246575344,14.28095890410959,14.374383561643837,14.467808219178083,14.56123287671233,14.654657534246576,14.748082191780824,14.84150684931507,14.934931506849317,15.028356164383563,15.12178082191781,15.215205479452056,15.308630136986302,15.40205479452055,15.495479452054797,15.588904109589043,15.68232876712329,15.775753424657536,15.869178082191782,15.962602739726028,16.056027397260277,16.149452054794523,16.24287671232877,16.336301369863016,16.429726027397262,16.523150684931508,16.616575342465755,16.71,16.803424657534247,16.896849315068494,16.99027397260274,17.083698630136986,17.177123287671233,17.27054794520548,17.36397260273973,17.457397260273975,17.55082191780822,17.644246575342468,17.737671232876714,17.83109589041096,17.83109589041096,17.924520547945207,18.017945205479453,18.1113698630137,18.204794520547946,18.298219178082192,18.39164383561644,18.485068493150685,18.57849315068493,18.671917808219177,18.765342465753427,18.858767123287674,18.95219178082192,19.045616438356166,19.139041095890413,19.23246575342466,19.325890410958905,19.41931506849315,19.512739726027398,19.606164383561644,19.699589041095894,19.79301369863014,19.886438356164387,19.979863013698633,20.07328767123288,20.166712328767126,20.260136986301372,20.35356164383562,20.446986301369865,20.54041095890411,20.633835616438358,20.727260273972604,20.82068493150685,20.914109589041097,21.007534246575347,21.100958904109593,21.19438356164384,21.287808219178086,21.381232876712332,21.474657534246578,21.568082191780825,21.66150684931507,21.754931506849317,21.848356164383564,21.94178082191781,22.035205479452056,22.128630136986303,22.22205479452055,22.315479452054795,22.408904109589045,22.50232876712329,22.595753424657538,22.689178082191784,22.78260273972603,22.876027397260277,22.969452054794523,23.06287671232877,23.156301369863016,23.249726027397262,23.34315068493151,23.436575342465755,23.53,23.623424657534247,23.716849315068497,23.810273972602744,23.90369863013699,23.997123287671236,24.090547945205483,24.18397260273973,24.277397260273975,24.277397260273975,24.37082191780822,24.464246575342468,24.557671232876714,24.65109589041096,24.744520547945207,24.837945205479453,24.9313698630137,25.02479452054795,25.118219178082196,25.211643835616442,25.30506849315069,25.398493150684935,25.49191780821918,25.585342465753428,25.678767123287674,25.77219178082192,25.865616438356167,25.959041095890413,26.05246575342466,26.145890410958906,26.239315068493152,26.332739726027402,26.426164383561648,26.519589041095895,26.61301369863014,26.706438356164387,26.799863013698634,26.89328767123288,26.986712328767126,27.080136986301373,27.17356164383562,27.266986301369865,27.36041095890411,27.453835616438358,27.547260273972604,27.64068493150685,27.7341095890411,27.827534246575347,27.920958904109593,28.01438356164384,28.107808219178086,28.201232876712332,28.29465753424658,28.388082191780825,28.48150684931507,28.574931506849317,28.668356164383564,28.76178082191781,28.855205479452056,28.948630136986303,29.042054794520553,29.1354794520548,29.228904109589045,29.32232876712329,29.415753424657538,29.509178082191784,29.60260273972603,29.696027397260277,29.789452054794523,29.88287671232877,29.976301369863016,30.069726027397262,30.16315068493151,30.256575342465755,30.350000000000005,30.44342465753425,30.536849315068498,30.630273972602744,30.72369863013699,30.817123287671237,30.910547945205483,31.00397260273973,31.097397260273976,31.190821917808222,31.28424657534247,31.377671232876715,31.47109589041096,31.564520547945207,31.657945205479457,31.751369863013704,31.84479452054795,31.938219178082196,32.03164383561644,32.125068493150685,32.218493150684935,32.31191780821918,32.40534246575343,32.49876712328767,32.59219178082192,32.68561643835616,32.77904109589041,32.872465753424656,32.965890410958906,33.059315068493156,33.1527397260274,33.24616438356165,33.33958904109589,33.43301369863014,33.526438356164384,33.619863013698634,33.71328767123288,33.80671232876713,33.90013698630137,33.99356164383562,34.08698630136986,34.18041095890411,34.273835616438355,34.367260273972605,34.460684931506854,34.5541095890411,34.64753424657535,34.74095890410959,34.83438356164384,34.92780821917808,35.02123287671233,35.11465753424658,35.208082191780825,35.301506849315075,35.39493150684932,35.48835616438357,35.58178082191781,35.67520547945206,35.7686301369863,35.86205479452055,35.955479452054796,36.048904109589046,36.14232876712329,36.23575342465754,36.32917808219179,36.42260273972603,36.51602739726028,36.609452054794524,36.702876712328774,36.796301369863016,36.889726027397266,36.98315068493151,37.07657534246576,37.17,37.26342465753425,37.356849315068494,37.450273972602744,37.54369863013699,37.63712328767124,37.73054794520549,37.82397260273973,37.91739726027398,38.01082191780822,38.10424657534247,38.197671232876715,38.291095890410965,38.38452054794521,38.47794520547946,38.5713698630137,38.66479452054795,38.75821917808219,38.85164383561644,38.94506849315069,39.038493150684936,39.131917808219185,39.22534246575343,39.31876712328768,39.41219178082192,39.50561643835617,39.59904109589041,39.69246575342466,39.785890410958906,39.879315068493156,39.9727397260274,40.06616438356165,40.15958904109589,40.25301369863014,40.34643835616439,40.439863013698634,40.533287671232884,40.62671232876713,40.72013698630138,40.81356164383562,40.90698630136987,41.00041095890411,41.09383561643836,41.187260273972605,41.280684931506855,41.3741095890411,41.46753424657535,41.56095890410959,41.65438356164384,41.74780821917809,41.84123287671233,41.93465753424658,42.028082191780825,42.121506849315075,42.21493150684932,42.30835616438357,42.40178082191781,42.49520547945206,42.5886301369863,42.68205479452055,42.775479452054796,42.868904109589046,42.962328767123296,43.05575342465754,43.14917808219179,43.24260273972603,43.33602739726028,43.429452054794524,43.522876712328774,43.61630136986302,43.70972602739727,43.80315068493151,43.89657534246576,43.99,44.08342465753425,44.176849315068495,44.270273972602745,44.363698630136994,44.45712328767124,44.55054794520549,44.64397260273973,44.73739726027398,44.83082191780822,44.92424657534247,45.017671232876715,45.111095890410965,45.20452054794521,45.29794520547946,45.3913698630137,45.48479452054795,45.57821917808219,45.67164383561644,45.76506849315069,45.858493150684936,45.951917808219186,46.04534246575343,46.13876712328768,46.23219178082192,46.32561643835617,46.419041095890414,46.512465753424664,46.60589041095891,46.699315068493156,46.7927397260274,46.88616438356165,46.9795890410959,47.07301369863014,47.16643835616439,47.259863013698634,47.353287671232884,47.44671232876713,47.54013698630138,47.63356164383562,47.72698630136987,47.82041095890411,47.91383561643836,48.007260273972605,48.100684931506855,48.1941095890411,48.28753424657535,48.3809589041096,48.47438356164384,48.56780821917809,48.66123287671233,48.75465753424658,48.848082191780826,48.941506849315076,49.03493150684932,49.12835616438357,49.22178082191781,49.31520547945206,49.408630136986304,49.502054794520554,49.5954794520548,49.688904109589046,49.782328767123296,49.87575342465754,49.96917808219179,50.06260273972603,50.15602739726028,50.249452054794524,50.342876712328774,50.43630136986302,50.52972602739727,50.62315068493151,50.71657534246576,50.81],\n",
       "\"..quantile..\":[0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "},\n",
       "\"stat\":\"identity\",\n",
       "\"color\":\"red\",\n",
       "\"size\":1.0,\n",
       "\"linetype\":\"dashed\",\n",
       "\"position\":\"identity\",\n",
       "\"xintercept\":19.785942622950824,\n",
       "\"geom\":\"vline\",\n",
       "\"data\":{\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"r8BFa0\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(tips.toMap()) { x = \"total_bill\" } +\n",
    "    geomHistogram(bins = 25, fill=\"white\", color=\"black\") { y = \"..density..\" } +\n",
    "    geomArea(stat = Stat.density(), fill = \"orange\", alpha = 0.2) +\n",
    "    geomVLine(xintercept = tips.total_bill.mean(), color=\"red\", linetype = \"dashed\", size = 1.0) +\n",
    "    ggtitle(\"Total bill amounts frequencies\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "26c632df-a1f5-4ca6-b30d-07d2043ef36d",
   "metadata": {},
   "source": [
    "This histogram shows that the average bill amount falls inside the range from 10 to 25 dollars, with it's mean located at about 20 dollars (red dashed line at 19.8).\n",
    "\n",
    "We can make the same plot, but with `tips` instead"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "545c184a-4515-48e2-9ac5-874de9ed8109",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"JM3721\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Tips amounts frequencies\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"tip\"\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..density..\"\n",
       "},\n",
       "\"stat\":\"bin\",\n",
       "\"bins\":25.0,\n",
       "\"color\":\"black\",\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"histogram\",\n",
       "\"fill\":\"white\",\n",
       "\"data\":{\n",
       "\"tip\":[0.93808,1.31824,1.6984,2.07856,2.45872,2.83888,3.2190399999999997,3.5991999999999997,3.97936,4.35952,4.73968,5.11984,5.5,5.88016,6.26032,6.64048,7.02064,7.4008,7.78096,8.16112,8.54128,8.92144,9.301599999999999,9.681759999999999,10.061919999999999],\n",
       "\"..density..\":[0.06830601092896174,0.22768670309653913,0.17076502732240437,0.5806010928961749,0.26183970856102,0.36429872495446264,0.2390710382513661,0.25045537340619306,0.18214936247723132,0.07969034608378871,0.03415300546448087,0.18214936247723132,0.022768670309653915,0.03415300546448087,0.0,0.04553734061930783,0.0,0.011384335154826957,0.0,0.0,0.0,0.011384335154826957,0.0,0.0,0.011384335154826957]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "},\n",
       "\"stat\":\"density\",\n",
       "\"alpha\":0.2,\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"area\",\n",
       "\"fill\":\"dark-green\",\n",
       "\"data\":{\n",
       "\"tip\":[1.0,1.0176125244618395,1.035225048923679,1.0528375733855186,1.0704500978473581,1.0880626223091976,1.1056751467710373,1.1232876712328768,1.1409001956947162,1.1585127201565557,1.1761252446183952,1.1937377690802349,1.2113502935420744,1.2289628180039138,1.2465753424657535,1.264187866927593,1.2818003913894325,1.299412915851272,1.3170254403131114,1.3346379647749511,1.3522504892367906,1.36986301369863,1.3874755381604698,1.4050880626223092,1.4227005870841487,1.4403131115459882,1.4579256360078277,1.4755381604696673,1.4931506849315068,1.5107632093933463,1.528375733855186,1.5459882583170255,1.563600782778865,1.5812133072407044,1.5988258317025439,1.6164383561643836,1.634050880626223,1.6516634050880625,1.6692759295499022,1.6868884540117417,1.7045009784735812,1.7221135029354206,1.7397260273972601,1.7573385518590998,1.7749510763209393,1.7925636007827788,1.8101761252446185,1.827788649706458,1.8454011741682974,1.8630136986301369,1.8806262230919764,1.898238747553816,1.9158512720156555,1.933463796477495,1.9510763209393347,1.9686888454011742,1.9863013698630136,2.003913894324853,2.003913894324853,2.0215264187866926,2.039138943248532,2.056751467710372,2.074363992172211,2.091976516634051,2.1095890410958904,2.12720156555773,2.1448140900195694,2.162426614481409,2.1800391389432487,2.1976516634050878,2.2152641878669277,2.232876712328767,2.2504892367906066,2.268101761252446,2.2857142857142856,2.303326810176125,2.3209393346379645,2.3385518590998045,2.3561643835616435,2.3737769080234834,2.391389432485323,2.4090019569471623,2.426614481409002,2.4442270058708413,2.461839530332681,2.4794520547945202,2.49706457925636,2.5146771037181996,2.532289628180039,2.5499021526418786,2.567514677103718,2.5851272015655575,2.602739726027397,2.620352250489237,2.637964774951076,2.655577299412916,2.6731898238747553,2.690802348336595,2.7084148727984343,2.7260273972602738,2.7436399217221137,2.7612524461839527,2.7788649706457926,2.796477495107632,2.796477495107632,2.8140900195694716,2.831702544031311,2.8493150684931505,2.86692759295499,2.8845401174168295,2.9021526418786694,2.9197651663405084,2.9373776908023483,2.954990215264188,2.9726027397260273,2.9902152641878668,3.0078277886497062,3.0254403131115457,3.043052837573385,3.060665362035225,3.0782778864970646,3.095890410958904,3.1135029354207435,3.131115459882583,3.1487279843444225,3.166340508806262,3.183953033268102,3.2015655577299413,3.219178082191781,3.2367906066536203,3.2544031311154598,3.2720156555772992,3.2896281800391387,3.307240704500978,3.3248532289628177,3.3424657534246576,3.360078277886497,3.3776908023483365,3.395303326810176,3.4129158512720155,3.430528375733855,3.4481409001956944,3.4657534246575343,3.483365949119374,3.5009784735812133,3.5185909980430528,3.5362035225048922,3.5538160469667317,3.571428571428571,3.5890410958904106,3.60665362035225,3.62426614481409,3.6418786692759295,3.659491193737769,3.6771037181996085,3.694716242661448,3.694716242661448,3.7123287671232874,3.729941291585127,3.747553816046967,3.7651663405088063,3.7827788649706457,3.8003913894324852,3.8180039138943247,3.835616438356164,3.8532289628180036,3.870841487279843,3.8884540117416826,3.9060665362035225,3.923679060665362,3.9412915851272015,3.958904109589041,3.9765166340508804,3.99412915851272,4.011741682974559,4.029354207436399,4.046966731898239,4.064579256360078,4.082191780821917,4.099804305283757,4.117416829745597,4.135029354207436,4.152641878669275,4.170254403131115,4.187866927592955,4.205479452054794,4.223091976516634,4.240704500978474,4.258317025440313,4.275929549902152,4.293542074363992,4.311154598825832,4.328767123287671,4.346379647749511,4.363992172211351,4.38160469667319,4.399217221135029,4.416829745596869,4.4344422700587085,4.4520547945205475,4.4696673189823874,4.487279843444227,4.504892367906066,4.522504892367905,4.540117416829745,4.557729941291585,4.575342465753424,4.592954990215264,4.610567514677104,4.628180039138943,4.645792563600782,4.663405088062622,4.681017612524462,4.698630136986301,4.71624266144814,4.73385518590998,4.75146771037182,4.769080234833659,4.786692759295499,4.804305283757339,4.821917808219178,4.839530332681017,4.857142857142857,4.874755381604697,4.892367906066536,4.909980430528376,4.9275929549902155,4.945205479452055,4.962818003913894,4.9804305283757335,4.998043052837573,5.0156555772994125,5.033268101761252,5.050880626223091,5.068493150684931,5.08610567514677,5.10371819960861,5.12133072407045,5.138943248532289,5.156555772994129,5.174168297455968,5.191780821917808,5.209393346379647,5.227005870841487,5.244618395303327,5.262230919765166,5.279843444227006,5.297455968688845,5.315068493150685,5.332681017612524,5.350293542074364,5.367906066536204,5.385518590998043,5.403131115459883,5.420743639921722,5.438356164383562,5.455968688845401,5.473581213307241,5.49119373776908,5.5088062622309195,5.526418786692759,5.5440313111545985,5.561643835616438,5.579256360078277,5.596868884540117,5.614481409001956,5.632093933463796,5.649706457925635,5.667318982387475,5.684931506849315,5.702544031311154,5.720156555772994,5.737769080234833,5.755381604696673,5.772994129158512,5.790606653620352,5.808219178082192,5.825831702544031,5.843444227005871,5.86105675146771,5.87866927592955,5.896281800391389,5.913894324853229,5.931506849315069,5.949119373776908,5.966731898238748,5.984344422700587,6.001956947162427,6.019569471624266,6.0371819960861055,6.0547945205479445,6.0724070450097845,6.090019569471624,6.107632093933463,6.125244618395303,6.142857142857142,6.160469667318982,6.178082191780821,6.195694716242661,6.2133072407045,6.23091976516634,6.24853228962818,6.266144814090019,6.283757338551859,6.301369863013698,6.318982387475538,6.336594911937377,6.354207436399217,6.371819960861057,6.389432485322896,6.407045009784736,6.424657534246575,6.442270058708415,6.459882583170254,6.477495107632094,6.495107632093934,6.512720156555773,6.5303326810176126,6.547945205479452,6.5655577299412915,6.5831702544031305,6.6007827788649704,6.6183953033268095,6.636007827788649,6.653620352250489,6.671232876712328,6.688845401174168,6.706457925636007,6.724070450097847,6.741682974559686,6.759295499021526,6.776908023483365,6.794520547945205,6.812133072407045,6.829745596868884,6.847358121330724,6.864970645792563,6.882583170254403,6.900195694716242,6.917808219178082,6.935420743639922,6.953033268101761,6.970645792563601,6.98825831702544,7.00587084148728,7.023483365949119,7.041095890410959,7.0587084148727985,7.076320939334638,7.0939334637964775,7.1115459882583165,7.129158512720156,7.1467710371819955,7.164383561643835,7.181996086105674,7.199608610567514,7.217221135029354,7.234833659491193,7.252446183953033,7.270058708414872,7.287671232876712,7.305283757338551,7.322896281800391,7.34050880626223,7.35812133072407,7.37573385518591,7.393346379647749,7.410958904109589,7.428571428571428,7.446183953033268,7.463796477495107,7.481409001956947,7.499021526418787,7.516634050880626,7.534246575342466,7.551859099804305,7.569471624266145,7.587084148727984,7.604696673189824,7.6223091976516635,7.6399217221135025,7.657534246575342,7.6751467710371815,7.692759295499021,7.71037181996086,7.7279843444227,7.745596868884539,7.763209393346379,7.780821917808219,7.798434442270058,7.816046966731898,7.833659491193737,7.851272015655577,7.868884540117416,7.886497064579256,7.904109589041095,7.921722113502935,7.939334637964775,7.956947162426614,7.974559686888454,7.992172211350293,8.009784735812133,8.027397260273972,8.04500978473581,8.062622309197652,8.08023483365949,8.097847358121331,8.11545988258317,8.13307240704501,8.150684931506849,8.168297455968688,8.185909980430528,8.203522504892367,8.221135029354208,8.238747553816047,8.256360078277886,8.273972602739725,8.291585127201564,8.309197651663403,8.326810176125244,8.344422700587085,8.362035225048924,8.379647749510763,8.397260273972602,8.414872798434441,8.43248532289628,8.450097847358121,8.46771037181996,8.4853228962818,8.50293542074364,8.520547945205479,8.538160469667318,8.555772994129157,8.573385518590998,8.590998043052837,8.608610567514678,8.626223091976517,8.643835616438356,8.661448140900195,8.679060665362034,8.696673189823874,8.714285714285714,8.731898238747554,8.749510763209393,8.767123287671232,8.784735812133071,8.80234833659491,8.819960861056751,8.83757338551859,8.855185909980431,8.87279843444227,8.89041095890411,8.908023483365948,8.925636007827787,8.943248532289628,8.960861056751467,8.978473581213308,8.996086105675147,9.013698630136986,9.031311154598825,9.048923679060666,9.066536203522505,9.084148727984344,9.101761252446183,9.119373776908024,9.136986301369863,9.154598825831702,9.17221135029354,9.189823874755382,9.20743639921722,9.22504892367906,9.2426614481409,9.26027397260274,9.277886497064578,9.295499021526417,9.313111545988258,9.330724070450097,9.348336594911936,9.365949119373777,9.383561643835616,9.401174168297455,9.418786692759294,9.436399217221135,9.454011741682974,9.471624266144813,9.489236790606654,9.506849315068493,9.524461839530332,9.542074363992171,9.559686888454012,9.57729941291585,9.59491193737769,9.61252446183953,9.63013698630137,9.647749510763209,9.665362035225048,9.682974559686889,9.700587084148728,9.718199608610567,9.735812133072407,9.753424657534246,9.771037181996086,9.788649706457925,9.806262230919765,9.823874755381604,9.841487279843443,9.859099804305282,9.876712328767123,9.894324853228962,9.911937377690801,9.929549902152642,9.947162426614481,9.96477495107632,9.98238747553816,10.0],\n",
       "\"..density..\":[0.08913956043553617,0.0933577533085389,0.09767635656042609,0.10209362319200661,0.10660769557844907,0.11121660810741749,0.11591828878555235,0.12071055963132064,0.1255911356505022,0.13055762216998137,0.13560751028805335,0.14073817018730236,0.14594684205138886,0.15123062433215462,0.15658645913038569,0.16201111448420566,0.16750116340503346,0.1730529595633391,0.17866260960552124,0.18432594217903076,0.19003847385434056,0.19579537225784055,0.20159141686675802,0.2074209580622784,0.21327787518639788,0.21915553449663366,0.225046748055673,0.23094373472448193,0.23683808454163974,0.24272072786299054,0.24858191069842825,0.2544111777117449,0.26019736434068536,0.2659285994438984,0.27159231978735876,0.27717529754354725,0.28266368179289253,0.28804305479011116,0.29329850349174286,0.298414706539919,0.30337603656758544,0.3081666773393961,0.3127707548790187,0.3171724813669132,0.32135631023300193,0.3253071005260207,0.32901028832632223,0.3324520626908949,0.3356195433877564,0.3385009574994534,0.34108581185881537,0.34336505822947844,0.34533124816198035,0.34697867454489534,0.34830349702851904,0.3493038487233281,0.3499799218619452,0.35033403045500444,0.35033403045500444,0.35037064835993054,0.35009642160758503,0.34952015428427075,0.34865276773448023,0.34750723332122846,0.34609847944426203,0.3444432739606062,0.34256008356643375,0.34046891207474855,0.3381911198519242,0.33574922695137616,0.3331667027000205,0.33046774464991274,0.3276770499026338,0.32481958184873966,0.3219203353412982,0.3190041032455847,0.316095247181781,0.31321747511051373,0.3103936282095152,0.307645479261331,0.3049935445243991,0.3024569108008707,0.30005307915099855,0.2977978264424898,0.2957050856692593,0.2937868457319543,0.29205307114593554,0.29051164193308887,0.2891683137629533,0.28802669823627275,0.28708826304896035,0.2863523516351536,0.2858162217619216,0.28547510243282437,0.28532226834984775,0.2853491310807358,0.2855453459787931,0.2858989338029286,0.28639641588572035,0.28702296159589635,0.28776254673910895,0.2885981214380897,0.2895117859320919,0.2904849726382776,0.2904849726382776,0.2914986327275121,0.2925334253873257,0.2935699078793237,0.2945887244507843,0.29557079213438364,0.29649748146893434,0.2973507902007729,0.2981135080817838,0.2987693709672959,0.2993032025355536,0.2997010420992488,0.2999502571570748,0.3000396395366544,0.29995948420577023,0.29970165007220856,0.29925960234860854,0.2986284363218223,0.2978048826307109,0.29678729441600066,0.2955756169551068,0.2941713406282476,0.2925774382748386,0.2907982881869162,0.28883958414583,0.28670823403724194,0.28441224867621306,0.28196062253828147,0.27936320812470006,0.27663058569163346,0.27377393004660083,0.27080487606358944,0.26773538449465195,0.26457760956419124,0.2613437697266472,0.2580460228528833,0.25469634698918625,0.2513064277089244,0.24788755295376141,0.2444505161414491,0.24100552820256313,0.2375621391005057,0.23412916928830105,0.23071465146217546,0.22732578288524413,0.22396888847370403,0.22064939476149067,0.21737181478579612,0.21413974386338028,0.21095586615464987,0.207821971837419,0.20473898463383175,0.20473898463383175,0.2017069993512692,0.19872532901065817,0.19579256104352524,0.19290662194299651,0.190064849654784,0.18726407289368996,0.1845006964713118,0.18177079162389942,0.1790701902383925,0.17639458179239045,0.1737396117531164,0.17110098012410818,0.16847453878903526,0.16585638628198685,0.16324295861465166,0.16063111481438896,0.15801821587403997,0.1554021958846104,0.1527816242152063,0.150155757719698,0.1475245820848393,0.14488884158769227,0.14225005669848662,0.1396105291453142,0.13697333424590416,0.134342300505498,0.1317219766748516,0.12911758665489445,0.12653497282106366,0.12398052851737884,0.12146112063493082,0.11898400333887174,0.11655672413996498,0.11418702361941162,0.11188273020763001,0.10965165148800751,0.10750146354486606,0.10543959990090948,0.10347314159356025,0.1016087099224447,0.0998523633627729,0.09820950008256628,0.09668476742693034,0.09528197964122863,0.09400404499859412,0.09285290337721029,0.09182947520078379,0.09093362251312,0.0901641228062497,0.08951865606266333,0.08899380530740444,0.08858507079659564,0.08828689779699156,0.08809271773798621,0.08799500234484008,0.08798533019150512,0.0880544649451875,0.08819244441464608,0.0883886793622665,0.08863206089827105,0.08891107514624653,0.08921392375466665,0.08952864873144245,0.08984325999983286,0.09014586401624085,0.09042479175524606,0.09066872435615242,0.09086681473949128,0.09100880354205229,0.09108512778539386,0.09108702078520126,0.09100660192656357,0.09083695507194452,0.0905721945324689,0.09020751771676958,0.08973924377213373,0.08916483774671839,0.08848292002541913,0.0876932610215243,0.08679676133727668,0.08579541783449661,0.08469227627705608,0.08349137141590997,0.0821976555804478,0.08081691701330461,0.07935568933605115,0.07782115365743919,0.07622103493177373,0.07456349424077607,0.07285701870696557,0.07111031074974694,0.06933217836739881,0.06753142807004314,0.06571676200209794,0.06389668067996569,0.06207939263457011,0.06027273209209687,0.058484085653525864,0.05672032874814823,0.05498777244230025,0.05329212098611447,0.051638440282285515,0.050031137265603,0.048473949994065686,0.046969948075180956,0.04552154288766129,0.04413050691180447,0.04279800135359263,0.04152461113969548,0.040310386274324136,0.039154888484997874,0.03805724204298028,0.03701618762520589,0.03603013808728336,0.0350972350405894,0.0342154051691557,0.03338241528230305,0.03259592517486007,0.03185353745617299,0.03115284360970762,0.030491465654493722,0.029867092895574036,0.029277513370598793,0.028720639721406864,0.02819452934060412,0.02769739876164077,0.02722763237471904,0.0267837856582006,0.026364583214405504,0.025968911988370837,0.025595810127060845,0.02524445200370985,0.0249141299866791,0.024604233573902765,0.024314226542395657,0.024043622777339083,0.023791961447113288,0.02355878217968486,0.023343600872567885,0.023145886733930643,0.022965041107268734,0.022800378577514897,0.022651110793756642,0.02251633337424676,0.022395016184577395,0.0222859972012693,0.02218798009217106,0.022099535563544596,0.02201910644309729,0.02194501639002511,0.021875482048801746,0.021808628394334395,0.021742506953431367,0.02167511653236875,0.021604426033616498,0.021528398907217278,0.021445018754443154,0.021352315583516912,0.021248392209505872,0.021131450292895936,0.02099981552355456,0.020851961478322748,0.020686531710689696,0.02050235966909426,0.02029848608541667,0.02007417352611319,0.0198289178540528,0.01956245640822846,0.019274772769885935,0.018966098045990473,0.01863690866310145,0.01828792072546208,0.01792008104932288,0.017534555040178063,0.017132711629795502,0.01671610553487286,0.01628645713820998,0.015845630325941627,0.015395608640272814,0.014938470126090954,0.014476361261732669,0.014011470369136969,0.013546000896820861,0.0130821449608969,0.012622057515133159,0.012167831501354247,0.011721474306877287,0.0112848858268097,0.010859838396585313,0.010447958824776437,0.01005071271870221,0.00966939125635099,0.009305100518328733,0.008958753453580985,0.008631064513121539,0.008322546947499605,0.008033512726755817,0.007764075006615292,0.0075141530320372,0.00728347933933084,0.007071609091139666,0.006877931354920542,0.006701682115282593,0.006541958793827155,0.006397736037030629,0.0062678825232715075,0.006151178534320073,0.0060463340344399305,0.005952007001616822,0.005866821760215999,0.00578938707242714,0.005718313757003393,0.005652231617825417,0.005589805481478933,0.005529750162051148,0.005470844192427303,0.0054119421841755866,0.005351985702297356,0.0052900125663196814,0.005225164515032593,0.005156693198228221,0.005083964484679874,0.005006461100906134,0.0049237836396031935,0.004835649999620103,0.004741893340638646,0.0046424586549746105,0.004537398075849419,0.004426865055840785,0.004311107560809094,0.004190460433264639,0.004065337084800468,0.0039362206798346806,0.0038036549725143976,0.003668234955320062,0.0035305974718189354,0.003391411937352636,0.0032513713004534233,0.003111183364762991,0.002971562576503618,0.002833222366484762,0.0026968681185952693,0.0025631908191235094,0.00243286142345624,0.002306525959117756,0.002184801367095777,0.0020682720673069017,0.0019574872192026096,0.0018529586351886198,0.0017551592929684902,0.0016645223833241165,0.0015814408223583586,0.0015062671519495872,0.0014393137491502128,0.0013808532645008598,0.0013311192106764166,0.0012903066264319432,0.0012585727463338581,0.0012360376140623916,0.001222784585938632,0.0012188606815146188,0.0012242767492952344,0.001239007427642611,0.0012629908933407801,0.0012961284028557263,0.0013382836436977204,0.0013892819251672447,0.0014489092488422686,0.0015169113091588924,0.0015929924830879675,0.0016768148749836487,0.0017679974879758352,0.0018661155966339773,0.001970700396925429,0.0020812390086505264,0.0021971749025334027,0.002317908819003205,0.002442800238488492,0.0025711694538910644,0.0027023002849748032,0.002835443461919558,0.0029698206915063975,0.003104629404617652,0.0032390481682782026,0.003372242729685158,0.003503372643938108,0.0036315984218635274,0.003756089119798241,0.003876030279814638,0.003990632116976148,0.00409913784011159,0.0042008319845627815,0.004295048629615205,0.004381179370037451,0.004458680910447601,0.00452708215314485,0.004585990654585308,0.004635098332768475,0.0046741863173034946,0.004703128845651395,0.004721896122749229,0.0047305560766274495,0.004729274959408122,0.004718316760867235,0.004698041420188464,0.004668901840250245,0.004631439727396869,0.004586280297783375,0.004534125908710959,0.004475748689571273,0.004411982261818365,0.004343712650551657,0.004271868501634902,0.004197410727654694,0.0041213217133483524,0.004044594216361819,0.003968220102337563,0.0038931790544253072,0.0038204273964351663,0.0037508871661263744,0.0036854355706802356,0.003624894950397364,0.0035700233692500132,0.003521505942279519,0.003479947000124388,0.0034458631803574147,0.0034196775239539656,0.003401714643247779,0.0033921970152834225,0.003391242441659183,0.0033988627028729104,0.0034149634219272185,0.0034393451386034793,0.0034717055824559817,0.003511643119287884,0.0035586613327314907,0.0036121746896566422,0.0036715152255713873,0.0037359401740693758,0.003804640452843179,0.0038767499079587586,0.003951355208123963,0.00402750627174237,0.0041042271017904,0.004180526897158565,0.004255411304222746,0.004327893669215436,0.004397006150590029,0.004461810551132374,0.004521408732161914,0.0045749524768351445,0.004621652676330536,0.004660787721527512,0.004691710993615123,0.004713857359753955,0.0047267485942936146,0.0047299976618981965,0.004723311815991252],\n",
       "\"..quantile..\":[0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "},\n",
       "\"stat\":\"identity\",\n",
       "\"color\":\"red\",\n",
       "\"size\":1.0,\n",
       "\"linetype\":\"dashed\",\n",
       "\"position\":\"identity\",\n",
       "\"xintercept\":2.9982786885245902,\n",
       "\"geom\":\"vline\",\n",
       "\"data\":{\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"JM3721\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(tips.toMap()) { x = \"tip\" } +\n",
    "    geomHistogram(bins = 25, fill=\"white\", color=\"black\") { y = \"..density..\" } +\n",
    "    geomArea(stat = Stat.density(), fill = \"dark-green\", alpha = 0.2) +\n",
    "    geomVLine(xintercept = tips.tip.mean(), color=\"red\", linetype = \"dashed\", size = 1.0) +\n",
    "    ggtitle(\"Tips amounts frequencies\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "3fcab728-de1e-4d95-8ecd-e8f8f13a3734",
   "metadata": {},
   "source": [
    "As shown above, the tips peak is at about two dollars, while the mean is right about at three dollars.\n",
    "\n",
    "It would be more interesting to see the distribution of the tips in relation to its total bill. "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "id": "8a52c949-f8d5-443b-af1e-d70e59a15aff",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":5,\"ncol\":8,\"columns\":[\"total_bill\",\"tip\",\"sex\",\"smoker\",\"day\",\"time\",\"size\",\"tip_pct\"],\"kotlin_dataframe\":[{\"total_bill\":16.99,\"tip\":1.01,\"sex\":\"Female\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.05944673337257211},{\"total_bill\":10.34,\"tip\":1.66,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":3,\"tip_pct\":0.16054158607350097},{\"total_bill\":21.01,\"tip\":3.5,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":3,\"tip_pct\":0.16658733936220846},{\"total_bill\":23.68,\"tip\":3.31,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.1397804054054054},{\"total_bill\":24.59,\"tip\":3.61,\"sex\":\"Female\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":4,\"tip_pct\":0.14680764538430255}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648130\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 5, columnsCount = 8</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"total_bill: Double\\\">total_bill</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">16.99</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">10.34</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">21.01</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">23.68</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">24.59</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip: Double\\\">tip</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.01</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.66</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.50</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.31</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.61</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"sex: String\\\">sex</span>\", children: [], rightAlign: false, values: [\"Female\",\"Male\",\"Male\",\"Male\",\"Female\"] }, \n",
       "{ name: \"<span title=\\\"smoker: Boolean\\\">smoker</span>\", children: [], rightAlign: false, values: [\"false\",\"false\",\"false\",\"false\",\"false\"] }, \n",
       "{ name: \"<span title=\\\"day: String\\\">day</span>\", children: [], rightAlign: false, values: [\"Sun\",\"Sun\",\"Sun\",\"Sun\",\"Sun\"] }, \n",
       "{ name: \"<span title=\\\"time: String\\\">time</span>\", children: [], rightAlign: false, values: [\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\"] }, \n",
       "{ name: \"<span title=\\\"size: Int\\\">size</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip_pct: Double\\\">tip_pct</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.059447</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.160542</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.166587</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.139780</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.146808</span></span>\"] }, \n",
       "], id: 553648130, rootId: 553648130, totalRows: 5 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648130) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 9,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "var data = tips.add(\"tip_pct\") { tip / total_bill }\n",
    "data.head()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "f2b6f0f5-cc85-45bc-a624-3e3525c58d25",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"AwjeyN\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Tips percentage on Total Bill amount\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"tip_pct\"\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[{\n",
       "\"aesthetic\":\"x\",\n",
       "\"format\":\".1%\"\n",
       "},{\n",
       "\"aesthetic\":\"x\",\n",
       "\"name\":\"Tips Percentage\"\n",
       "}],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..density..\"\n",
       "},\n",
       "\"stat\":\"bin\",\n",
       "\"bins\":25.0,\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"histogram\",\n",
       "\"fill\":\"gray\",\n",
       "\"tooltips\":{\n",
       "\"variables\":[\"tip_pct\"],\n",
       "\"formats\":[{\n",
       "\"field\":\"tip_pct\",\n",
       "\"format\":\".1%\"\n",
       "}]\n",
       "},\n",
       "\"data\":{\n",
       "\"tip_pct\":[0.030996153812219693,0.05949576447109982,0.08799537512997996,0.11649498578886008,0.1449945964477402,0.17349420710662034,0.20199381776550046,0.2304934284243806,0.2589930390832607,0.2874926497421409,0.315992260401021,0.34449187105990114,0.37299148171878127,0.4014910923776614,0.4299907030365415,0.45849031369542165,0.4869899243543018,0.5154895350131818,0.543989145672062,0.5724887563309421,0.6009883669898223,0.6294879776487023,0.6579875883075825,0.6864871989664626,0.7149868096253428],\n",
       "\"..density..\":[0.15185712198880535,1.2148569759104428,3.0371424397761064,5.6187135135857975,10.174427173249958,6.985427611485045,6.074284879552213,2.27785682983208,0.6074284879552214,0.455571365966416,0.15185712198880535,0.0,0.0,0.0,0.15185712198880535,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.15185712198880535]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "},\n",
       "\"stat\":\"identity\",\n",
       "\"color\":\"red\",\n",
       "\"size\":1.0,\n",
       "\"linetype\":\"dashed\",\n",
       "\"position\":\"identity\",\n",
       "\"xintercept\":0.16080258172250478,\n",
       "\"geom\":\"vline\",\n",
       "\"data\":{\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"AwjeyN\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(data.toMap()) { x=\"tip_pct\" } +\n",
    "    geomHistogram(\n",
    "        bins = 25,\n",
    "        fill=\"gray\",\n",
    "        tooltips = layerTooltips(\"tip_pct\")\n",
    "            .format(\"tip_pct\", \".1%\")\n",
    "    ) { y = \"..density..\" } +\n",
    "    geomVLine(\n",
    "        xintercept = data.tip_pct.mean(),\n",
    "        linetype = \"dashed\",\n",
    "        color = \"red\",\n",
    "        size = 1.0,\n",
    "    ) +\n",
    "    scaleXContinuous(format = \".1%\") +\n",
    "    xlab(\"Tips Percentage\") +\n",
    "    ggtitle(\"Tips percentage on Total Bill amount\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "df9257d1-2d15-4d7f-bfd0-2cf82ee7f8e2",
   "metadata": {},
   "source": [
    "We can see that the peak is at about 15% of the total bill. We can spot also some outliers, and let's see their details in the dataframe."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "id": "c1f32306-735d-4b52-b2af-f3713309eb8b",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":5,\"ncol\":8,\"columns\":[\"total_bill\",\"tip\",\"sex\",\"smoker\",\"day\",\"time\",\"size\",\"tip_pct\"],\"kotlin_dataframe\":[{\"total_bill\":7.25,\"tip\":5.15,\"sex\":\"Male\",\"smoker\":true,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.710344827586207},{\"total_bill\":9.6,\"tip\":4.0,\"sex\":\"Female\",\"smoker\":true,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.4166666666666667},{\"total_bill\":3.07,\"tip\":1.0,\"sex\":\"Female\",\"smoker\":true,\"day\":\"Sat\",\"time\":\"Dinner\",\"size\":1,\"tip_pct\":0.32573289902280134},{\"total_bill\":11.61,\"tip\":3.39,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sat\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.29198966408268734},{\"total_bill\":23.17,\"tip\":6.5,\"sex\":\"Male\",\"smoker\":true,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":4,\"tip_pct\":0.2805351747949935}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648131\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 5, columnsCount = 8</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"total_bill: Double\\\">total_bill</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">7.25</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">9.60</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.07</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">11.61</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">23.17</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip: Double\\\">tip</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">5.15</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4.00</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.00</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.39</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">6.50</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"sex: String\\\">sex</span>\", children: [], rightAlign: false, values: [\"Male\",\"Female\",\"Female\",\"Male\",\"Male\"] }, \n",
       "{ name: \"<span title=\\\"smoker: Boolean\\\">smoker</span>\", children: [], rightAlign: false, values: [\"true\",\"true\",\"true\",\"false\",\"true\"] }, \n",
       "{ name: \"<span title=\\\"day: String\\\">day</span>\", children: [], rightAlign: false, values: [\"Sun\",\"Sun\",\"Sat\",\"Sat\",\"Sun\"] }, \n",
       "{ name: \"<span title=\\\"time: String\\\">time</span>\", children: [], rightAlign: false, values: [\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\"] }, \n",
       "{ name: \"<span title=\\\"size: Int\\\">size</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip_pct: Double\\\">tip_pct</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.710345</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.416667</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.325733</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.291990</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.280535</span></span>\"] }, \n",
       "], id: 553648131, rootId: 553648131, totalRows: 5 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648131) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "data.sortBy { tip_pct.desc() }.head(5)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "5ba6d3da-e040-4251-8445-3ce0b862eaa2",
   "metadata": {},
   "source": [
    "It can also be interesting to analyze the amount of money spent by each person inside a group"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "id": "81003cd7-e13f-4d2d-bb78-818df01756ad",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":5,\"ncol\":9,\"columns\":[\"total_bill\",\"tip\",\"sex\",\"smoker\",\"day\",\"time\",\"size\",\"tip_pct\",\"bill_pp\"],\"kotlin_dataframe\":[{\"total_bill\":16.99,\"tip\":1.01,\"sex\":\"Female\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.05944673337257211,\"bill_pp\":8.495},{\"total_bill\":10.34,\"tip\":1.66,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":3,\"tip_pct\":0.16054158607350097,\"bill_pp\":3.4466666666666668},{\"total_bill\":21.01,\"tip\":3.5,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":3,\"tip_pct\":0.16658733936220846,\"bill_pp\":7.003333333333334},{\"total_bill\":23.68,\"tip\":3.31,\"sex\":\"Male\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":2,\"tip_pct\":0.1397804054054054,\"bill_pp\":11.84},{\"total_bill\":24.59,\"tip\":3.61,\"sex\":\"Female\",\"smoker\":false,\"day\":\"Sun\",\"time\":\"Dinner\",\"size\":4,\"tip_pct\":0.14680764538430255,\"bill_pp\":6.1475}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648132\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 5, columnsCount = 9</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"total_bill: Double\\\">total_bill</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">16.99</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">10.34</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">21.01</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">23.68</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">24.59</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip: Double\\\">tip</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.01</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1.66</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.50</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.31</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.61</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"sex: String\\\">sex</span>\", children: [], rightAlign: false, values: [\"Female\",\"Male\",\"Male\",\"Male\",\"Female\"] }, \n",
       "{ name: \"<span title=\\\"smoker: Boolean\\\">smoker</span>\", children: [], rightAlign: false, values: [\"false\",\"false\",\"false\",\"false\",\"false\"] }, \n",
       "{ name: \"<span title=\\\"day: String\\\">day</span>\", children: [], rightAlign: false, values: [\"Sun\",\"Sun\",\"Sun\",\"Sun\",\"Sun\"] }, \n",
       "{ name: \"<span title=\\\"time: String\\\">time</span>\", children: [], rightAlign: false, values: [\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\",\"Dinner\"] }, \n",
       "{ name: \"<span title=\\\"size: Int\\\">size</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"tip_pct: Double\\\">tip_pct</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.059447</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.160542</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.166587</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.139780</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.146808</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"bill_pp: Double\\\">bill_pp</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">8.495000</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3.446667</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">7.003333</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">11.840000</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">6.147500</span></span>\"] }, \n",
       "], id: 553648132, rootId: 553648132, totalRows: 5 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648132) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "// adding Bill Per Person col\n",
    "data = data.add(\"bill_pp\") { total_bill / size }\n",
    "data.head()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "088f116c-4291-41e3-a1f7-94c7b21a8ccb",
   "metadata": {},
   "source": [
    "And similarly as above:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "07431bb8-5479-48ca-a4aa-3c613590e139",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"1ECuQ4\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Distribution of Bill per Person\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"bill_pp\"\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[{\n",
       "\"aesthetic\":\"x\",\n",
       "\"name\":\"Bill per Person\"\n",
       "}],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"y\":\"..density..\"\n",
       "},\n",
       "\"stat\":\"bin\",\n",
       "\"bins\":25.0,\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"histogram\",\n",
       "\"fill\":\"gray\",\n",
       "\"data\":{\n",
       "\"bill_pp\":[2.7552879999999997,3.490264,4.225239999999999,4.960215999999999,5.695192,6.430168,7.165144,7.900119999999999,8.635095999999999,9.370071999999999,10.105048,10.840024,11.575,12.309975999999999,13.044952,13.779928,14.514904,15.24988,15.984855999999999,16.719832,17.454808,18.189784,18.92476,19.659736,20.394712],\n",
       "\"..density..\":[0.01177689843602789,0.02944224609006972,0.06477294139815339,0.1354343320143207,0.17665347654041832,0.21787262106651595,0.12954588279630677,0.15309967966836255,0.12365743357829283,0.07654983983418127,0.09421518748822312,0.04121914452609761,0.04121914452609761,0.04710759374411156,0.01177689843602789,0.017665347654041832,0.017665347654041832,0.005888449218013945,0.01177689843602789,0.017665347654041832,0.005888449218013945,0.0,0.0,0.0,0.005888449218013945]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "},\n",
       "\"stat\":\"identity\",\n",
       "\"color\":\"red\",\n",
       "\"size\":1.0,\n",
       "\"linetype\":\"dashed\",\n",
       "\"position\":\"identity\",\n",
       "\"xintercept\":7.88822950819673,\n",
       "\"geom\":\"vline\",\n",
       "\"data\":{\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"1ECuQ4\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(data.toMap()) { x=\"bill_pp\" } +\n",
    "    geomHistogram(\n",
    "        bins = 25,\n",
    "        fill=\"gray\",\n",
    "    ) { y = \"..density..\" } +\n",
    "    geomVLine(\n",
    "        xintercept = data[\"bill_pp\"].cast<Double>().mean(),\n",
    "        linetype = \"dashed\",\n",
    "        color = \"red\",\n",
    "        size = 1.0,\n",
    "    ) +\n",
    "    xlab(\"Bill per Person\") +\n",
    "    ggtitle(\"Distribution of Bill per Person\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "018428e2-79c4-4cb9-ab6f-b8c63954744a",
   "metadata": {},
   "source": [
    "It can be useful to see the bill per person with `total_bill` in the same plot."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "b5cc3882-c82d-4e03-a019-4607fb9f80c6",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"qiO36R\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Total bill amount and bill per person distributions\"\n",
       "},\n",
       "\"mapping\":{\n",
       "},\n",
       "\"data\":{\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"x\":\"total_bill\",\n",
       "\"y\":\"..density..\"\n",
       "},\n",
       "\"stat\":\"bin\",\n",
       "\"bins\":25.0,\n",
       "\"color\":\"white\",\n",
       "\"alpha\":0.2,\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"histogram\",\n",
       "\"fill\":\"blue\",\n",
       "\"data\":{\n",
       "\"..density..\":[0.00213745731497742,0.0,0.010687286574887099,0.01709965851981936,0.05129897555945808,0.061986262134345174,0.04916151824448066,0.06626117676430002,0.04916151824448066,0.04916151824448066,0.02564948777972904,0.0320618597246613,0.01923711583479678,0.01709965851981936,0.01709965851981936,0.00854982925990968,0.01496220120484194,0.0,0.00854982925990968,0.00641237194493226,0.00213745731497742,0.00427491462995484,0.0,0.00641237194493226,0.00213745731497742],\n",
       "\"total_bill\":[2.5452072,4.5699816,6.594756,8.6195304,10.6443048,12.6690792,14.6938536,16.718628000000002,18.7434024,20.7681768,22.7929512,24.817725600000003,26.8425,28.8672744,30.8920488,32.9168232,34.9415976,36.966372,38.9911464,41.0159208,43.0406952,45.0654696,47.090244000000006,49.115018400000004,51.1397928]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "\"x\":\"total_bill\"\n",
       "},\n",
       "\"stat\":\"density\",\n",
       "\"color\":\"blue\",\n",
       "\"size\":1.0,\n",
       "\"position\":\"identity\",\n",
       "\"geom\":\"line\",\n",
       "\"data\":{\n",
       "\"..density..\":[0.0019605901168061705,0.0020609544948427126,0.002166420138196568,0.002277243284184698,0.0023936871943235094,0.0025160215735436854,0.002644521924879366,0.0027794688414775827,0.002921147238273116,0.003069845526150482,0.0032258547318641697,0.003389467567404557,0.003560977452876687,0.003740677497298047,0.003928859442016786,0.004125812571701669,0.004331822598057264,0.004547170521573303,0.004772131476724364,0.005006973566097566,0.005251956688943401,0.005507331369619677,0.005773337591335598,0.006050203640505287,0.006338144966890884,0.006637363064561416,0.006948044378517098,0.007270359241637008,0.007604460846403234,0.00795048425564456,0.008308545456328286,0.008678740460217355,0.00906114445500175,0.009455811009314889,0.009862771334855476,0.010282033608658446,0.010713582358393304,0.011157377913416632,0.011613355924165038,0.012081426952346104,0.012561476134264192,0.013053362919503993,0.013556920887083459,0.014071957641076181,0.01459825478758719,0.015135567994841417,0.01568362713800737,0.016242136530224688,0.016810775241129113,0.017389197503971592,0.017977033212199595,0.018573888506115844,0.019179346449938867,0.01979296779927014,0.020414291858615825,0.021042837428222123,0.02167810383906062,0.022319572074346043,0.02296670597548548,0.023618953529851237,0.024275748237237237,0.024936510551312973,0.025600649391827525,0.026267563722748907,0.02693664419095573,0.027607274819531912,0.028278834749161128,0.028950700020578597,0.02962224539052022,0.03029284617311975,0.030961880098247116,0.031628729177862976,0.03229278157108683,0.03295343343834937,0.033610090774718465,0.03426217121227134,0.03490910578121707,0.03555034061937346,0.03618533861956857,0.03681358100456404,0.037434568819200854,0.03804782432964237,0.03865289231983686,0.0392493412756481,0.03983676444750689,0.040414780782921184,0.040983035720750224,0.04154120183979932,0.04208897935502608,0.042626096455472086,0.043152309478935946,0.04366740291939735,0.04417118926427139,0.044663508659726577,0.045144228403526915,0.04561324226615925,0.04607046964237168,0.04651585453666739,0.04694936438776802,0.04737098873856132,0.04778073775957349,0.048178640635539664,0.048564743826165345,0.048939109213672255,0.04930181215116635,0.049652939427251595,0.04999258716360535,0.05032085866341501,0.050637862229629585,0.05094370897287551,0.05123851062961274,0.05123851062961274,0.051522377411633985,0.05179541590832787,0.052057727063213516,0.0523094042460991,0.052550531441807394,0.052781181575742527,0.05300141499563678,0.0532112781276189,0.05341080232328529,0.0536000029127474,0.05377887847668047,0.05394741034823322,0.05410556235329069,0.05425328079504776,0.0543904946861704,0.05451711622903595,0.05463304154168035,0.05473815162419018,0.05483231355738678,0.05491538192281249,0.054987200430275635,0.05504760373659797,0.05509641943675646,0.05513347020637621,0.05515857607254399,0.055171556788193805,0.05517223428391894,0.05516043516998754,0.05513599326061773,0.05509875209221355,0.055048567407274165,0.05498530957608159,0.054908865929027814,0.05481914297356526,0.05471606847122337,0.054599593351922,0.05446969344488922,0.05432637100783479,0.05416965603859894,0.05399960735625096,0.05381631344151146,0.05361989302937088,0.05341049544982713,0.05318830071572151,0.05295351935966937,0.052706392025012615,0.052447188818521645,0.05217620843520958,0.051893777068045675,0.051893777068045675,0.05160024711754565,0.05129599571813404,0.05098142309980879,0.0506569508049582,0.05032301978118625,0.04998008837168111,0.04962863022501419,0.049269132146290674,0.04890209191129639,0.048528016064717176,0.048147417722666466,0.04776081439866938,0.04736872587094428,0.04697167210733409,0.04657017126259201,0.04616473776096684,0.045755880475191336,0.04534410101109101,0.04492989210513764,0.04451373614040248,0.044096103784555744,0.04367745275183773,0.04325822668931842,0.042838854186294614,0.042419747904360365,0.04200130382454464,0.04158390060695052,0.041167899057559236,0.04075364169627826,0.040341452419919065,0.039931636253572614,0.03952447918380606,0.03912024806721394,0.03871919060810717,0.03832153539949181,0.03792749202196042,0.03753725119566414,0.03715098498113494,0.03676884702535883,0.036390972850141305,0.036017480180433104,0.035648469310877125,0.035284023509377836,0.03492420945696678,0.03456907772362779,0.03421866328004078,0.03387298604540063,0.03353205147155464,0.03319585116368683,0.032864363537650924,0.03253755451382983,0.03221537824707789,0.03189777789189708,0.03158468640151916,0.03127602735902784,0.030971715838071585,0.030671659290106856,0.030375758454489806,0.03008390828711682,0.02979599890272199,0.029511916525384797,0.029231544441301698,0.028954763947445426,0.028681455289387113,0.028411498581299894,0.028144774701007992,0.027881166152898793,0.027620557891579613,0.027362838099340432,0.027362838099340432,0.02710789891077697,0.02685563707833062,0.026605954573011636,0.026358759115179778,0.026113964630952956,0.02587149163059185,0.02563126750605112,0.02539322674578499,0.025157311065831533,0.024923469457161007,0.024691658150244565,0.02446184049876448,0.024233986785332444,0.024008073952991163,0.02378408526713626,0.02356200991329415,0.023341842536917703,0.02312358273200409,0.022907234485888387,0.022692805588016132,0.02248030701084186,0.022269752271234046,0.022061156780887645,0.02185453719425367,0.021649910762389948,0.021447294700923034,0.02124670557999024,0.02104815874360972,0.02085166776541121,0.020657243947059603,0.0204648958650249,0.020274628970607896,0.020086445247328964,0.019900342928941873,0.01971631628045411,0.019534355443634986,0.019354446347582918,0.019176570684016417,0.019000705946062516,0.018826825528451847,0.018654898886204082,0.018484891748110584,0.01831676638060454,0.018150481896959312,0.017985994606185118,0.017823258395505727,0.017662225139901257,0.01750284513190111,0.01734506752461005,0.017188840780851023,0.017034113121311593,0.016880832964688425,0.016728949353032416,0.016578412355804577,0.016429173446555003,0.016281185846627702,0.016134404830868455,0.015988787990959964,0.01584429545272238,0.01570089004448639,0.01555853741445921,0.015417206095850922,0.015276867519396166,0.015137495973781997,0.0149990685153647,0.014861564829411634,0.014724967045928666,0.01458925951391358,0.01445442853860198,0.014320462086930521,0.01418734946702374,0.014055080988004745,0.013923647606828175,0.013793040569128364,0.013663251051261755,0.013534269810793783,0.01340608685263668,0.013278691117881764,0.013152070202091196,0.013026210109420426,0.012901095048438938,0.012776707274908266,0.012653026986072319,0.012530032270222216,0.012407699114429818,0.012286001472411214,0.012164911393497721,0.01204439921267202,0.011924433800586432,0.011804982871433825,0.011686013345507746,0.011567491762281149,0.011449384738870996,0.01133165946785377,0.011214284247570622,0.011097229037324699,0.010980466029240698,0.010863970228039576,0.01074772002959053,0.010631697788845906,0.010515890367649118,0.010400289652936833,0.010284893036033974,0.010169703844067644,0.010054731714996549,0.009939992908366078,0.009825510544644568,0.00971131476686709,0.009597442819296456,0.009483939038893555,0.009370854756556683,0.009258248106323263,0.00914618374201222,0.009034732462098311,0.008923970744935932,0.008813980197763771,0.008704846924208075,0.008596660816237282,0.008489514777687304,0.00838350388755557,0.008278724512235871,0.008175273376719592,0.008073246605508445,0.007972738744556786,0.007873841775979645,0.007776644137517408,0.00768122975883577,0.007587677126658084,0.007496058390476916,0.007406438520175909,0.007318874526318164,0.007233414753130777,0.007150098253348458,0.007068954253084518,0.006990001713789584,0.006913248997154174,0.006838693637527434,0.006766322225080897,0.006696110401561434,0.006628022969072905,0.00656201411092045,0.006498027722165516,0.006435997846192968,0.006375849212302187,0.006317497868120192,0.006260851899511964,0.00620581222964663,0.006152273487980192,0.006100124939147569,0.006049251461127309,0.005999534561558094,0.005950853420752001,0.005903085949767061,0.005856109851871351,0.005809803675850398,0.005764047849874745,0.005718725685049111,0.005673724338300134,0.0056289357249172094,0.005584257371828309,0.005539593203558198,0.005494854253765793,0.005449959296276674,0.00540483539060034,0.005359418338034529,0.005313653045594889,0.005267493796151824,0.005220904424292168,0.005173858398536408,0.0051263388116180465,0.005078338281557304,0.0050298587672237055,0.004980911302970348,0.0049315156577263404,0.004881699924644652,0.004831500048013082,0.004780959294640441,0.004730127677324515,0.004679061338290178,0.004627821900654402,0.004576475796030465,0.004525093576328797,0.00447374921765022,0.0044225194239040574,0.004371482937425453,0.004320719863420538,0.004270311014544265,0.0042203372813231724,0.004170879033484908,0.0041220155565593825,0.00407382452738465,0.004026381531396264,0.0039797596238142,0.003934028936078627,0.0038892563281370373,0.00384550508646193,0.0038028346669916353,0.0037613004815469754,0.003720953725693051,0.003681841245496512,0.003644005440181572,0.0036074841973186135,0.0035723108568920174,0.003538514200391763,0.003506118460958161,0.0034751433505805637,0.0034456041004078076,0.003417511510367151,0.0033908720045056594,0.0033656876887567843,0.0033419564081891037,0.003319671801204819,0.003298823348613946,0.0032793964160059363,0.0032613722883630307,0.0032447281963979236,0.0032294373346407946,0.00321546887183609,0.0032027879547263036,0.0031913557067875234,0.0031811292239294535,0.0031720615695713665,0.0031641017718466113,0.0031571948259643393,0.0031512817049620056,0.003146299382211192,0.003142180869089124,0.003138855271197316,0.0031362478663969173,0.0031342802077391203,0.003132870254101514,0.003131932531002099,0.0031313783236578473,0.003131115903891568,0.0031310507919778646,0.003131086053965615,0.0031311226344310624,0.0031310597240130347,0.0031307951604714313,0.0031302258614032685,0.003129248286158763,0.003127758923934189,0.0031256548044892338,0.0031228340274542,0.003119196305765684,0.0031146435184062115,0.0031090802673305715,0.003102414433244527,0.0030945577247642785,0.003085426215429933,0.0030749408630742804,0.0030630280061587412,0.003049619831879476,0.0030346548111147447,0.0030180780956247763,0.0029998418733217062,0.0029799056778922277,0.002958236649571437,0.0029348097444240424,0.0029096078900791816,0.002882622086477684,0.0028538514508157768,0.0028233032064968458,0.0027909926165231975,0.0027569428623633635,0.0027211848699084736,0.002683757084675427,0.002644705198917685,0.0026040818337601515,0.0025619461798773177,0.00251836360057962,0.00247340520145843,0.002427147370963492,0.0023796712964470747,0.002331062460306883,0.002281410120896268,0.0022308067828477863,0.002179347661377914,0.002127130145010656,0.0020742532609805864,0.0020208171473568176,0.0019669225356736993,0.0019126702475682196],\n",
       "\"total_bill\":[2.875,2.96880626223092,3.0626125244618394,3.1564187866927593,3.2502250489236792,3.3440313111545987,3.4378375733855187,3.531643835616438,3.625450097847358,3.719256360078278,3.8130626223091975,3.9068688845401174,4.000675146771037,4.094481409001957,4.188287671232876,4.282093933463797,4.375900195694716,4.469706457925636,4.563512720156556,4.657318982387475,4.751125244618395,4.844931506849315,4.938737769080235,5.032544031311154,5.126350293542075,5.220156555772994,5.313962818003914,5.407769080234834,5.501575342465753,5.595381604696673,5.689187866927593,5.782994129158513,5.876800391389432,5.970606653620353,6.064412915851272,6.1582191780821915,6.252025440313112,6.345831702544031,6.439637964774951,6.533444227005871,6.627250489236791,6.72105675146771,6.814863013698631,6.90866927592955,7.0024755381604695,7.09628180039139,7.190088062622309,7.283894324853229,7.377700587084149,7.471506849315069,7.565313111545988,7.6591193737769085,7.752925636007828,7.8467318982387475,7.940538160469668,8.034344422700588,8.128150684931507,8.221956947162427,8.315763209393346,8.409569471624266,8.503375733855187,8.597181996086107,8.690988258317025,8.784794520547946,8.878600782778864,8.972407045009785,9.066213307240705,9.160019569471626,9.253825831702544,9.347632093933465,9.441438356164383,9.535244618395303,9.629050880626224,9.722857142857144,9.816663405088063,9.910469667318983,10.004275929549902,10.098082191780822,10.191888454011742,10.285694716242663,10.379500978473581,10.473307240704502,10.56711350293542,10.66091976516634,10.754726027397261,10.848532289628182,10.9423385518591,11.03614481409002,11.129951076320939,11.22375733855186,11.31756360078278,11.411369863013698,11.505176125244619,11.59898238747554,11.692788649706458,11.786594911937378,11.880401174168298,11.974207436399217,12.068013698630137,12.161819960861058,12.255626223091976,12.349432485322897,12.443238747553817,12.537045009784736,12.630851272015656,12.724657534246576,12.818463796477495,12.912270058708415,13.006076320939336,13.099882583170254,13.193688845401175,13.193688845401175,13.287495107632095,13.381301369863014,13.475107632093934,13.568913894324854,13.662720156555773,13.756526418786693,13.850332681017614,13.944138943248532,14.037945205479453,14.131751467710373,14.225557729941292,14.319363992172212,14.413170254403132,14.506976516634051,14.600782778864971,14.694589041095892,14.78839530332681,14.88220156555773,14.976007827788651,15.06981409001957,15.16362035225049,15.25742661448141,15.351232876712329,15.44503913894325,15.53884540117417,15.632651663405088,15.726457925636009,15.820264187866929,15.914070450097848,16.007876712328766,16.101682974559687,16.195489236790607,16.289295499021527,16.383101761252448,16.476908023483368,16.57071428571429,16.66452054794521,16.758326810176126,16.852133072407046,16.945939334637966,17.039745596868883,17.133551859099803,17.227358121330724,17.321164383561644,17.414970645792565,17.508776908023485,17.602583170254405,17.696389432485326,17.790195694716246,17.790195694716246,17.884001956947163,17.977808219178083,18.071614481409004,18.16542074363992,18.25922700587084,18.35303326810176,18.44683953033268,18.540645792563602,18.634452054794522,18.728258317025443,18.822064579256363,18.91587084148728,19.0096771037182,19.10348336594912,19.19728962818004,19.29109589041096,19.384902152641878,19.4787084148728,19.57251467710372,19.66632093933464,19.76012720156556,19.85393346379648,19.947739726027397,20.041545988258317,20.135352250489237,20.229158512720158,20.32296477495108,20.416771037182,20.510577299412915,20.604383561643836,20.698189823874756,20.791996086105677,20.885802348336597,20.979608610567517,21.073414872798434,21.167221135029354,21.261027397260275,21.354833659491195,21.448639921722116,21.542446183953036,21.636252446183953,21.730058708414873,21.823864970645793,21.917671232876714,22.011477495107634,22.105283757338555,22.19909001956947,22.29289628180039,22.386702544031312,22.480508806262232,22.574315068493153,22.668121330724073,22.76192759295499,22.85573385518591,22.94954011741683,23.04334637964775,23.13715264187867,23.230958904109592,23.32476516634051,23.41857142857143,23.51237769080235,23.60618395303327,23.69999021526419,23.79379647749511,23.887602739726027,23.981409001956948,24.075215264187868,24.16902152641879,24.26282778864971,24.26282778864971,24.35663405088063,24.450440313111546,24.544246575342466,24.638052837573387,24.731859099804307,24.825665362035227,24.919471624266148,25.013277886497065,25.107084148727985,25.200890410958905,25.294696673189826,25.388502935420746,25.482309197651666,25.576115459882583,25.669921722113504,25.763727984344424,25.857534246575344,25.951340508806265,26.045146771037185,26.138953033268102,26.232759295499022,26.326565557729943,26.420371819960863,26.514178082191783,26.607984344422704,26.70179060665362,26.79559686888454,26.88940313111546,26.98320939334638,27.077015655577302,27.17082191780822,27.26462818003914,27.35843444227006,27.45224070450098,27.5460469667319,27.63985322896282,27.733659491193738,27.827465753424658,27.92127201565558,28.0150782778865,28.10888454011742,28.20269080234834,28.296497064579256,28.390303326810177,28.484109589041097,28.577915851272017,28.671722113502938,28.765528375733858,28.859334637964775,28.953140900195695,29.046947162426616,29.140753424657536,29.234559686888456,29.328365949119377,29.422172211350293,29.515978473581214,29.609784735812134,29.703590998043055,29.797397260273975,29.891203522504895,29.985009784735812,30.078816046966733,30.172622309197653,30.266428571428573,30.360234833659494,30.454041095890414,30.54784735812133,30.64165362035225,30.73545988258317,30.829266144814092,30.923072407045012,31.016878669275933,31.11068493150685,31.20449119373777,31.29829745596869,31.39210371819961,31.48590998043053,31.57971624266145,31.673522504892368,31.76732876712329,31.86113502935421,31.95494129158513,32.04874755381605,32.14255381604697,32.23636007827788,32.33016634050881,32.423972602739724,32.51777886497065,32.611585127201565,32.70539138943249,32.799197651663405,32.893003913894326,32.986810176125246,33.08061643835617,33.17442270058709,33.26822896281801,33.36203522504893,33.45584148727984,33.54964774951077,33.64345401174168,33.73726027397261,33.83106653620352,33.92487279843444,34.01867906066536,34.11248532289628,34.206291585127204,34.300097847358124,34.393904109589045,34.48771037181996,34.581516634050885,34.6753228962818,34.769129158512726,34.86293542074364,34.95674168297456,35.05054794520548,35.1443542074364,35.23816046966732,35.33196673189824,35.42577299412916,35.51957925636008,35.613385518591,35.70719178082192,35.80099804305284,35.894804305283756,35.98861056751468,36.0824168297456,36.17622309197652,36.27002935420744,36.36383561643836,36.45764187866928,36.5514481409002,36.64525440313112,36.73906066536204,36.83286692759296,36.92667318982388,37.02047945205479,37.114285714285714,37.208091976516634,37.301898238747555,37.395704500978475,37.489510763209395,37.583317025440316,37.677123287671236,37.77092954990216,37.86473581213308,37.958542074364,38.05234833659492,38.14615459882583,38.23996086105675,38.33376712328767,38.42757338551859,38.52137964774951,38.61518590998043,38.70899217221135,38.80279843444227,38.896604696673194,38.990410958904114,39.084217221135034,39.17802348336595,39.27182974559687,39.36563600782779,39.45944227005871,39.55324853228963,39.64705479452055,39.74086105675147,39.83466731898239,39.92847358121331,40.02227984344423,40.11608610567515,40.20989236790607,40.303698630136985,40.397504892367905,40.491311154598826,40.585117416829746,40.67892367906067,40.77272994129159,40.86653620352251,40.96034246575343,41.05414872798435,41.14795499021527,41.24176125244619,41.33556751467711,41.42937377690802,41.52318003913894,41.61698630136986,41.71079256360078,41.804598825831704,41.898405088062624,41.992211350293545,42.086017612524465,42.179823874755385,42.273630136986306,42.367436399217226,42.461242661448146,42.55504892367906,42.64885518590998,42.7426614481409,42.83646771037182,42.93027397260274,43.02408023483366,43.11788649706458,43.2116927592955,43.30549902152642,43.39930528375734,43.49311154598826,43.586917808219184,43.6807240704501,43.77453033268102,43.86833659491194,43.96214285714286,44.05594911937378,44.1497553816047,44.24356164383562,44.33736790606654,44.43117416829746,44.52498043052838,44.6187866927593,44.71259295499022,44.806399217221134,44.900205479452055,44.994011741682975,45.087818003913895,45.181624266144816,45.275430528375736,45.36923679060666,45.46304305283758,45.5568493150685,45.65065557729942,45.74446183953034,45.83826810176126,45.93207436399217,46.02588062622309,46.11968688845401,46.21349315068493,46.30729941291585,46.40110567514677,46.494911937377694,46.588718199608614,46.682524461839535,46.776330724070455,46.870136986301375,46.963943248532296,47.05774951076321,47.15155577299413,47.24536203522505,47.33916829745597,47.43297455968689,47.52678082191781,47.62058708414873,47.71439334637965,47.80819960861057,47.90200587084149,47.99581213307241,48.08961839530333,48.183424657534246,48.27723091976517,48.37103718199609,48.46484344422701,48.55864970645793,48.65245596868885,48.74626223091977,48.84006849315069,48.93387475538161,49.02768101761253,49.12148727984345,49.21529354207437,49.30909980430528,49.402906066536204,49.496712328767124,49.590518590998045,49.684324853228965,49.778131115459885,49.871937377690806,49.965743639921726,50.059549902152646,50.15335616438357,50.24716242661449,50.34096868884541,50.43477495107632,50.52858121330724,50.62238747553816,50.71619373776908,50.81],\n",
       "\"..quantile..\":[0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "\"x\":\"bill_pp\",\n",
       "\"y\":\"..density..\"\n",
       "},\n",
       "\"stat\":\"bin\",\n",
       "\"bins\":25.0,\n",
       "\"color\":\"white\",\n",
       "\"alpha\":0.2,\n",
       "\"position\":\"stack\",\n",
       "\"geom\":\"histogram\",\n",
       "\"fill\":\"red\",\n",
       "\"data\":{\n",
       "\"bill_pp\":[2.5452072,4.5699816,6.594756,8.6195304,10.6443048,12.6690792,14.6938536,16.718628000000002,18.7434024,20.7681768,22.7929512,24.817725600000003,26.8425,28.8672744,30.8920488,32.9168232,34.9415976,36.966372,38.9911464,41.0159208,43.0406952,45.0654696,47.090244000000006,49.115018400000004,51.1397928],\n",
       "\"..density..\":[0.00854982925990968,0.0961855791739839,0.17954641445810327,0.11542269500878068,0.06626117676430002,0.0320618597246613,0.00854982925990968,0.01282474388986452,0.0,0.00213745731497742,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0]\n",
       "}\n",
       "},{\n",
       "\"mapping\":{\n",
       "\"x\":\"bill_pp\"\n",
       "},\n",
       "\"stat\":\"density\",\n",
       "\"color\":\"red\",\n",
       "\"size\":1.0,\n",
       "\"position\":\"identity\",\n",
       "\"geom\":\"line\",\n",
       "\"data\":{\n",
       "\"bill_pp\":[2.875,2.96880626223092,3.0626125244618394,3.1564187866927593,3.2502250489236792,3.3440313111545987,3.4378375733855187,3.531643835616438,3.625450097847358,3.719256360078278,3.8130626223091975,3.9068688845401174,4.000675146771037,4.094481409001957,4.188287671232876,4.282093933463797,4.375900195694716,4.469706457925636,4.563512720156556,4.657318982387475,4.751125244618395,4.844931506849315,4.938737769080235,5.032544031311154,5.126350293542075,5.220156555772994,5.313962818003914,5.407769080234834,5.501575342465753,5.595381604696673,5.689187866927593,5.689187866927593,5.782994129158513,5.876800391389432,5.970606653620353,6.064412915851272,6.1582191780821915,6.252025440313112,6.345831702544031,6.439637964774951,6.533444227005871,6.627250489236791,6.72105675146771,6.814863013698631,6.90866927592955,7.0024755381604695,7.09628180039139,7.190088062622309,7.283894324853229,7.283894324853229,7.377700587084149,7.471506849315069,7.565313111545988,7.6591193737769085,7.752925636007828,7.8467318982387475,7.940538160469668,8.034344422700588,8.128150684931507,8.221956947162427,8.315763209393346,8.409569471624266,8.503375733855187,8.597181996086107,8.690988258317025,8.784794520547946,8.878600782778864,8.972407045009785,9.066213307240705,9.160019569471626,9.253825831702544,9.347632093933465,9.347632093933465,9.441438356164383,9.535244618395303,9.629050880626224,9.722857142857144,9.816663405088063,9.910469667318983,10.004275929549902,10.098082191780822,10.191888454011742,10.285694716242663,10.379500978473581,10.473307240704502,10.56711350293542,10.66091976516634,10.754726027397261,10.848532289628182,10.9423385518591,11.03614481409002,11.129951076320939,11.22375733855186,11.31756360078278,11.411369863013698,11.505176125244619,11.59898238747554,11.692788649706458,11.786594911937378,11.880401174168298,11.974207436399217,12.068013698630137,12.161819960861058,12.255626223091976,12.349432485322897,12.443238747553817,12.537045009784736,12.630851272015656,12.724657534246576,12.818463796477495,12.912270058708415,13.006076320939336,13.099882583170254,13.193688845401175,13.287495107632095,13.381301369863014,13.475107632093934,13.568913894324854,13.662720156555773,13.756526418786693,13.850332681017614,13.944138943248532,14.037945205479453,14.131751467710373,14.225557729941292,14.319363992172212,14.413170254403132,14.506976516634051,14.600782778864971,14.694589041095892,14.78839530332681,14.88220156555773,14.976007827788651,15.06981409001957,15.16362035225049,15.25742661448141,15.351232876712329,15.44503913894325,15.53884540117417,15.632651663405088,15.726457925636009,15.820264187866929,15.914070450097848,16.007876712328766,16.101682974559687,16.195489236790607,16.289295499021527,16.383101761252448,16.476908023483368,16.57071428571429,16.66452054794521,16.758326810176126,16.852133072407046,16.945939334637966,17.039745596868883,17.133551859099803,17.227358121330724,17.321164383561644,17.414970645792565,17.508776908023485,17.602583170254405,17.696389432485326,17.790195694716246,17.884001956947163,17.977808219178083,18.071614481409004,18.16542074363992,18.25922700587084,18.35303326810176,18.44683953033268,18.540645792563602,18.634452054794522,18.728258317025443,18.822064579256363,18.91587084148728,19.0096771037182,19.10348336594912,19.19728962818004,19.29109589041096,19.384902152641878,19.4787084148728,19.57251467710372,19.66632093933464,19.76012720156556,19.85393346379648,19.947739726027397,20.041545988258317,20.135352250489237,20.229158512720158,20.32296477495108,20.416771037182,20.510577299412915,20.604383561643836,20.698189823874756,20.791996086105677,20.885802348336597,20.979608610567517,21.073414872798434,21.167221135029354,21.261027397260275,21.354833659491195,21.448639921722116,21.542446183953036,21.636252446183953,21.730058708414873,21.823864970645793,21.917671232876714,22.011477495107634,22.105283757338555,22.19909001956947,22.29289628180039,22.386702544031312,22.480508806262232,22.574315068493153,22.668121330724073,22.76192759295499,22.85573385518591,22.94954011741683,23.04334637964775,23.13715264187867,23.230958904109592,23.32476516634051,23.41857142857143,23.51237769080235,23.60618395303327,23.69999021526419,23.79379647749511,23.887602739726027,23.981409001956948,24.075215264187868,24.16902152641879,24.26282778864971,24.35663405088063,24.450440313111546,24.544246575342466,24.638052837573387,24.731859099804307,24.825665362035227,24.919471624266148,25.013277886497065,25.107084148727985,25.200890410958905,25.294696673189826,25.388502935420746,25.482309197651666,25.576115459882583,25.669921722113504,25.763727984344424,25.857534246575344,25.951340508806265,26.045146771037185,26.138953033268102,26.232759295499022,26.326565557729943,26.420371819960863,26.514178082191783,26.607984344422704,26.70179060665362,26.79559686888454,26.88940313111546,26.98320939334638,27.077015655577302,27.17082191780822,27.26462818003914,27.35843444227006,27.45224070450098,27.5460469667319,27.63985322896282,27.733659491193738,27.827465753424658,27.92127201565558,28.0150782778865,28.10888454011742,28.20269080234834,28.296497064579256,28.390303326810177,28.484109589041097,28.577915851272017,28.671722113502938,28.765528375733858,28.859334637964775,28.953140900195695,29.046947162426616,29.140753424657536,29.234559686888456,29.328365949119377,29.422172211350293,29.515978473581214,29.609784735812134,29.703590998043055,29.797397260273975,29.891203522504895,29.985009784735812,30.078816046966733,30.172622309197653,30.266428571428573,30.360234833659494,30.454041095890414,30.54784735812133,30.64165362035225,30.73545988258317,30.829266144814092,30.923072407045012,31.016878669275933,31.11068493150685,31.20449119373777,31.29829745596869,31.39210371819961,31.48590998043053,31.57971624266145,31.673522504892368,31.76732876712329,31.86113502935421,31.95494129158513,32.04874755381605,32.14255381604697,32.23636007827788,32.33016634050881,32.423972602739724,32.51777886497065,32.611585127201565,32.70539138943249,32.799197651663405,32.893003913894326,32.986810176125246,33.08061643835617,33.17442270058709,33.26822896281801,33.36203522504893,33.45584148727984,33.54964774951077,33.64345401174168,33.73726027397261,33.83106653620352,33.92487279843444,34.01867906066536,34.11248532289628,34.206291585127204,34.300097847358124,34.393904109589045,34.48771037181996,34.581516634050885,34.6753228962818,34.769129158512726,34.86293542074364,34.95674168297456,35.05054794520548,35.1443542074364,35.23816046966732,35.33196673189824,35.42577299412916,35.51957925636008,35.613385518591,35.70719178082192,35.80099804305284,35.894804305283756,35.98861056751468,36.0824168297456,36.17622309197652,36.27002935420744,36.36383561643836,36.45764187866928,36.5514481409002,36.64525440313112,36.73906066536204,36.83286692759296,36.92667318982388,37.02047945205479,37.114285714285714,37.208091976516634,37.301898238747555,37.395704500978475,37.489510763209395,37.583317025440316,37.677123287671236,37.77092954990216,37.86473581213308,37.958542074364,38.05234833659492,38.14615459882583,38.23996086105675,38.33376712328767,38.42757338551859,38.52137964774951,38.61518590998043,38.70899217221135,38.80279843444227,38.896604696673194,38.990410958904114,39.084217221135034,39.17802348336595,39.27182974559687,39.36563600782779,39.45944227005871,39.55324853228963,39.64705479452055,39.74086105675147,39.83466731898239,39.92847358121331,40.02227984344423,40.11608610567515,40.20989236790607,40.303698630136985,40.397504892367905,40.491311154598826,40.585117416829746,40.67892367906067,40.77272994129159,40.86653620352251,40.96034246575343,41.05414872798435,41.14795499021527,41.24176125244619,41.33556751467711,41.42937377690802,41.52318003913894,41.61698630136986,41.71079256360078,41.804598825831704,41.898405088062624,41.992211350293545,42.086017612524465,42.179823874755385,42.273630136986306,42.367436399217226,42.461242661448146,42.55504892367906,42.64885518590998,42.7426614481409,42.83646771037182,42.93027397260274,43.02408023483366,43.11788649706458,43.2116927592955,43.30549902152642,43.39930528375734,43.49311154598826,43.586917808219184,43.6807240704501,43.77453033268102,43.86833659491194,43.96214285714286,44.05594911937378,44.1497553816047,44.24356164383562,44.33736790606654,44.43117416829746,44.52498043052838,44.6187866927593,44.71259295499022,44.806399217221134,44.900205479452055,44.994011741682975,45.087818003913895,45.181624266144816,45.275430528375736,45.36923679060666,45.46304305283758,45.5568493150685,45.65065557729942,45.74446183953034,45.83826810176126,45.93207436399217,46.02588062622309,46.11968688845401,46.21349315068493,46.30729941291585,46.40110567514677,46.494911937377694,46.588718199608614,46.682524461839535,46.776330724070455,46.870136986301375,46.963943248532296,47.05774951076321,47.15155577299413,47.24536203522505,47.33916829745597,47.43297455968689,47.52678082191781,47.62058708414873,47.71439334637965,47.80819960861057,47.90200587084149,47.99581213307241,48.08961839530333,48.183424657534246,48.27723091976517,48.37103718199609,48.46484344422701,48.55864970645793,48.65245596868885,48.74626223091977,48.84006849315069,48.93387475538161,49.02768101761253,49.12148727984345,49.21529354207437,49.30909980430528,49.402906066536204,49.496712328767124,49.590518590998045,49.684324853228965,49.778131115459885,49.871937377690806,49.965743639921726,50.059549902152646,50.15335616438357,50.24716242661449,50.34096868884541,50.43477495107632,50.52858121330724,50.62238747553816,50.71619373776908,50.81],\n",
       "\"..density..\":[0.017162666444205367,0.019433355855681112,0.02189294987915382,0.02454893661850238,0.027409737376066473,0.030484762081251112,0.033784314751440704,0.03731931881318737,0.041100842714733105,0.045139420738516824,0.04944418156232333,0.05402181666240918,0.05887544032947829,0.06400341072364547,0.06939819472275746,0.07504536618350567,0.08092282602963535,0.08700032256889839,0.0932393320014469,0.09959333380690613,0.10600848626922264,0.11242467726644971,0.11877689832231256,0.12499686913964267,0.13101482781099244,0.1367613996063739,0.1421694640023685,0.14717595320724844,0.15172353245629705,0.15576212889817315,0.15925028835492455,0.15925028835492455,0.1621563450670804,0.16445938783422145,0.16614999775079364,0.16723072082511045,0.16771622723605392,0.16763310236239437,0.16701921704767447,0.16592263846351388,0.1644000689500242,0.1625148365182561,0.16033450326316942,0.1579282011096193,0.15536384175745205,0.1527053734454951,0.15001026666928127,0.14732740191747395,0.14469550510346005,0.14469550510346005,0.14214223349930297,0.13968396156523624,0.1373262583369854,0.1350649925248624,0.13288795407482148,0.13077684608181353,0.1287094811513365,0.12666201207094568,0.12461103668423273,0.12253543856111584,0.12041785514991983,0.11824570020147539,0.11601170434288448,0.11371397429213413,0.11135560546570088,0.10894391317462729,0.10648937294461731,0.1040043794330033,0.10150194457690344,0.09899445766526992,0.09649262198569936,0.09400466426461176,0.09400466426461176,0.09153588507146489,0.08908858275282933,0.08666234362790652,0.08425465141667997,0.08186173391872195,0.07947953926931692,0.07710472110995764,0.07473551353597166,0.072372392555803,0.07001844884126386,0.06767943291458771,0.0653634736877163,0.06308050923428085,0.060841500135628765,0.058657517177451164,0.05653880466953891,0.05449391804465039,0.052529021069836083,0.05064740656924332,0.048849278270119305,0.04713180362316996,0.045489421251899746,0.04391436443163368,0.042397345263160496,0.04092833377251551,0.03949736215126567,0.03809528636510363,0.03671444468176716,0.0353491643668375,0.033996082782329136,0.032654266183584496,0.03132512730766816,0.030012159947345966,0.0287205236493386,0.02745652310353158,0.026227033605658596,0.025038925523674205,0.023898536902816056,0.022811234779689236,0.0217810936268718,0.020810705266281874,0.01990112044545422,0.019051909869878605,0.018261323269475552,0.017526519898095266,0.016843842854269948,0.016209112194081523,0.015617916842485537,0.015065891340049915,0.014548969016165626,0.014063607069426085,0.013606980589428882,0.013177141738331524,0.012773137718569708,0.012395077865389277,0.012044137546937596,0.011722485816236139,0.011433125899169895,0.011179643034984763,0.010965862659830357,0.010795432529451207,0.010671353673682866,0.01059549532089115,0.01056813636983879,0.010587579150125101,0.010649879174227508,0.010748727135804168,0.010875507120831985,0.011019539158126708,0.011168496673502604,0.011308972237976738,0.011427150302021388,0.011509535148494093,0.011543677269529519,0.011518842243867736,0.011426572659051127,0.01126110469329373,0.011019615134156346,0.010702290082780947,0.010312221586109402,0.009855151409264318,0.009339090986551366,0.00877385267846055,0.008170529751111377,0.007540961375204434,0.006897215122771208,0.0062511137862740545,0.005613826720643689,0.004995539059083847,0.004405205631938057,0.00385039057182835,0.0033371885868491956,0.0028702197751623087,0.0024526866317711557,0.002086479554309317,0.0017723157142525584,0.0015098956840303656,0.0012980627739898713,0.0011349516852017872,0.001018115802714944,9.446261075426711E-4,9.111390125344489E-4,9.139350387963083E-4,9.489346604522744E-4,0.00101170132877457,0.001097444142612577,0.0012010334659368598,0.001317041785755483,0.0014398192623674374,0.0015636090064095843,0.001682701617253103,0.0017916226138284912,0.0018853408611343178,0.001959481716080113,0.002010526036990888,0.0020359758513997512,0.002034469490514616,0.00200583318480397,0.001951061978009297,0.001872229625745458,0.0017723340403360578,0.0016550909646377244,0.0015246931672866452,0.0013855550402226938,0.0012420628281129928,0.001098348917416898,9.581050242825092E-4,8.24444311754328E-4,6.998171094984329E-4,5.859796757646711E-4,4.8401091121804036E-4,3.9436852577588575E-4,3.169740936372438E-4,2.513157304052522E-4,1.965576408018486E-4,1.5164724470197825E-4,1.1541265722694978E-4,8.664563159004633E-5,6.416736619585903E-5,4.687659548619485E-5,3.3780977816961215E-5,2.401389867301417E-5,1.6839431818677257E-5,1.1648405908191557E-5,7.94840265567283E-6,5.350169301203785E-6,3.5524624142856624E-6,2.3268359201943197E-6,1.503406170855439E-6,9.582111025002626E-7,6.024491136213958E-7,3.736404117099039E-7,2.2859230940106946E-7,1.3795698342742234E-7,8.212966754263293E-8,4.823149694452563E-8,2.7940599394632114E-8,1.596669380919063E-8,9.000540208920044E-9,5.004912127519611E-9,2.745355806656525E-9,1.4855084305690633E-9,7.929136684389415E-10,4.1749472149552477E-10,2.1684548026272886E-10,1.1110255876118889E-10,5.6152881713354353E-11,2.799589380134303E-11,1.3768636266126321E-11,6.679774749802971E-12,3.196737749117917E-12,1.5091294777745356E-12,7.0278146019069E-13,3.228407821518584E-13,1.4629543904497682E-13,6.539544260248485E-14,2.8836230033293783E-14,1.254306810094151E-14,5.381996117496099E-15,2.2780186802595133E-15,9.511422706857364E-16,3.917490807081364E-16,1.5916397984937643E-16,6.379048366901461E-17,2.521978042598607E-17,9.835603394837578E-18,3.783859623857163E-18,1.4359632502471568E-18,5.375587502067492E-19,1.9851020905629172E-19,7.231261366205802E-20,2.598481158690264E-20,9.210843140603188E-21,3.2207239415486665E-21,1.1109177808600522E-21,3.7799376417216813E-22,1.2687079276405513E-22,4.2006159102049926E-23,1.3719509771835334E-23,4.420165339602745E-24,1.404794302577132E-24,4.404141467036977E-25,1.3620218826475142E-25,4.155097419781251E-26,1.2504105727511817E-26,3.7119179236909725E-27,1.0869720909416686E-27,3.139877631391819E-28,8.94708067847147E-29,2.5149206400823322E-29,6.973350873133728E-30,1.9073616728733948E-30,5.146344872349848E-31,1.3697427865517368E-31,3.596279676925484E-32,9.314127884078802E-33,2.3796069866484047E-33,5.997117738344425E-34,1.4909195723252954E-34,3.656285986637624E-35,8.845052211588924E-36,2.1107409933416252E-36,4.968711601323078E-37,1.1537904387648914E-37,2.6429222230214615E-38,5.971949557292945E-39,1.3311352365022257E-39,2.926863912076874E-40,6.348296185787823E-41,1.3582700767116377E-41,2.8667471290191402E-42,5.9685239432251135E-43,1.2257976815666576E-43,2.4833901362782256E-44,4.9630129575356366E-45,9.784083880687235E-46,1.9026952177263133E-46,3.6499976675248397E-47,6.907012600023109E-48,1.2893244267582144E-48,2.3741517470079057E-49,4.312499308179658E-50,7.72723115090891E-51,1.3658187495733463E-51,2.3814229671321914E-52,4.0959467913403643E-53,6.94938505255631E-54,1.1630884589514244E-54,1.9202307404805138E-55,3.1272916878121397E-56,5.0240929328251377E-57,7.961982868253249E-58,1.2446840217096065E-58,1.9194256367126384E-59,2.919831360830013E-60,4.3814569369070845E-61,6.485651780319319E-62,9.470284110872007E-63,1.3641015108823075E-63,1.938227150284963E-64,2.716670436278796E-65,3.756155312526192E-66,5.123000844021653E-67,6.892545065855383E-68,9.147640788622746E-69,1.1976030442250785E-69,1.5466462085459756E-70,1.9703499731410135E-71,2.4761107426272933E-72,3.06952416985274E-73,3.753585834892614E-74,4.527890823898724E-75,5.3879039228671095E-76,6.324381213653529E-77,7.323025360177056E-78,8.364448996827305E-79,9.424502578294966E-80,1.0474995349873598E-80,1.1484802762805414E-81,1.2421314112352121E-82,1.325213491946364E-83,1.3946924635351558E-84,1.4479226454122595E-85,1.4828136248813714E-86,1.4979664146999778E-87,1.4927665189493524E-88,1.467425278867222E-89,1.4229656214686383E-90,1.3611535503136245E-91,1.2843817732046818E-92,1.1955161530172208E-93,1.0977187034440353E-94,9.942623189053333E-96,8.883522327849752E-97,7.829674666285305E-98,6.807325801017641E-99,5.838263030437435E-100,4.939296294356801E-101,4.122121626077142E-102,3.3935232231436264E-103,2.755847303182263E-104,2.207668069086263E-105,1.7445631834630483E-106,1.3599217850443663E-107,1.0457201098293869E-108,7.93215678493491E-110,5.935282854519341E-111,4.380925347283354E-112,3.189808314134836E-113,2.291066102213017E-114,1.623248292647559E-115,1.1345054612381352E-116,7.821724732694361E-118,5.319523919060056E-119,3.5687596929490015E-120,2.3617620797124457E-121,1.5418044205234274E-122,9.92879922519709E-124,6.307227148380431E-125,3.952341988539935E-126,2.4431203517119133E-127,1.4897366689589206E-128,8.960834475656206E-130,5.316939314485961E-131,3.112069253352684E-132,1.7968470180796902E-133,1.0234043406957492E-134,5.74986578145945E-136,3.1867093938452003E-137,1.7422140633019525E-138,9.395823945938991E-140,4.9985325799096586E-141,2.6231582088570026E-142,1.3579405066674328E-143,6.934438532844265E-145,3.4931413807158622E-146,1.7357825786016265E-147,8.508417416023568E-149,4.114116178720634E-150,1.9623596996577188E-151,9.233258235809256E-153,4.2855409455552335E-154,1.9621428251534155E-155,8.861962041600461E-157,3.948239193273588E-158,1.7352071406972075E-159,7.522695761459031E-161,3.2171400782403646E-162,1.3571902935777982E-163,5.64788467152836E-165,2.3184898338549544E-166,9.388557554223436E-168,3.750307263184778E-169,1.4777777727663667E-170,5.744149177323257E-172,2.2025034190124366E-173,8.330705336082737E-175,3.108288034192089E-176,1.1440237505229677E-177,4.153585225049136E-179,1.4875977477689358E-180,5.255598734826841E-182,1.831610766407493E-183,6.296779376858162E-185,2.1353942419312933E-186,7.143514961591405E-188,2.3573287399431572E-189,7.673661618665196E-191,2.464106298668303E-192,7.805316791338032E-194,2.438910878151485E-195,7.517537933936401E-197,2.285754728023678E-198,6.855796631397756E-200,2.0284323438701135E-201,5.920214311549392E-203,1.7044671324319363E-204,4.8407665027589954E-206,1.356169395084491E-207,3.7479004343931267E-209,1.0217307209082097E-210,2.747635756549845E-212,7.288802258738667E-214,1.907337194779624E-215,4.9234903244394864E-217,1.25369819505259E-218,3.149105521946874E-220,7.802894361725977E-222,1.907210118725359E-223,4.598494359726625E-225,1.0937223201629682E-226,2.5660953698266236E-228,5.9389929813646735E-230,1.3558983493970714E-231,3.0536253555045478E-233,6.783888092282476E-235,1.486674520249067E-236,3.2138634628121406E-238,6.853513103594189E-240,1.4416949380988086E-241,2.991629456685631E-243,6.123736978720158E-245,1.2365154815318904E-246,2.462957333354136E-248,4.839366566253665E-250,9.379818871600531E-252,1.793389758737323E-253,3.382433132073714E-255,6.293004685104649E-257,1.1549447032767623E-258,2.0909258359164177E-260,3.7341380299955066E-262,6.578341373084296E-264,1.143185605251671E-265,1.9597081049715514E-267,3.3139071137728155E-269,5.527943236834162E-271,9.0962234320554E-273,1.4764983469080884E-274,2.364172229829758E-276,3.734217065403285E-278,5.818275854131127E-280,8.942590783574672E-282,1.3558345869655575E-283,2.0277964583162537E-285,2.9916880931879125E-287,4.353941271843374E-289,6.2506203506090704E-291,8.851930132544136E-293,1.2365939981048025E-294,1.7040824045588418E-296,2.3164789126596045E-298,3.1062786451806406E-300,4.1089113552759797E-302,5.361513871856344E-304,6.901164776038701E-306,8.762573241521225E-308,1.097527032989303E-309,1.3560418477287E-311,1.6527427444E-313,1.987063417E-315,2.356636E-317],\n",
       "\"..quantile..\":[0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.25,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.5,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,0.75,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"qiO36R\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 14,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(data.toMap()) +\n",
    "    geomHistogram(\n",
    "        bins=25, fill=\"blue\", \n",
    "        color=\"white\", alpha=0.2) {\n",
    "            x=\"total_bill\" ; y=\"..density..\"\n",
    "    } +\n",
    "    geomLine(stat = Stat.density(), color=\"blue\", size=1.0) {\n",
    "        x = \"total_bill\"\n",
    "    } +\n",
    "    geomHistogram(bins=25, fill=\"red\", color=\"white\", alpha=0.2) {\n",
    "        x=\"bill_pp\" ; y=\"..density..\"\n",
    "    } +\n",
    "    geomLine(stat = Stat.density(), color=\"red\", size=1.0) { x=\"bill_pp\"} +\n",
    "    ggtitle(\"Total bill amount and bill per person distributions\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "ef34d7d1-b983-40df-be88-c4c4a07d3edb",
   "metadata": {},
   "source": [
    "We want to see if there is correlation with smokers, group size and tip percentage:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "id": "09049aaa-f334-496a-8695-2b373332c839",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":6,\"ncol\":2,\"columns\":[\"size\",\"smoker\"],\"kotlin_dataframe\":[{\"size\":1,\"smoker\":{\"false\":0.15982946957504365,\"true\":0.27475456139951254}},{\"size\":2,\"smoker\":{\"false\":0.16499567702041978,\"true\":0.16670580270902313}},{\"size\":3,\"smoker\":{\"false\":0.14967110695957225,\"true\":0.15754264158846806}},{\"size\":4,\"smoker\":{\"false\":0.14760448860982958,\"true\":0.14203604842767492}},{\"size\":5,\"smoker\":{\"false\":0.17841521012935002,\"true\":0.08611590893452556}},{\"size\":6,\"smoker\":{\"false\":0.1562292007202838,\"true\":null}}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648133\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 6, columnsCount = 2</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"size: Int\\\">size</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">5</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">6</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"false: Double\\\">false</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.159829</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.164996</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.149671</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.147604</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.178415</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.156229</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"true: Double?\\\">true</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.274755</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.166706</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.157543</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.142036</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.086116</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"null\\\">null</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"smoker: DataRow\\\">smoker</span>\", children: [1, 2], rightAlign: false, values: [\"<span class=\\\"formatted\\\" title=\\\"false: 0.15982946957504365\\ntrue: 0.27475456139951254\\\"><span class=\\\"structural\\\">{ </span><span class=\\\"structural\\\">false: </span><span class=\\\"numbers\\\">0.2</span><span class=\\\"structural\\\">, </span><span class=\\\"structural\\\">true: </span><span class=\\\"numbers\\\">0.3</span><span class=\\\"structural\\\"> }</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"false: 0.16499567702041978\\ntrue: 0.16670580270902313\\\"><span class=\\\"structural\\\">{ </span><span class=\\\"structural\\\">false: </span><span class=\\\"numbers\\\">0.2</span><span class=\\\"structural\\\">, </span><span class=\\\"structural\\\">true: </span><span class=\\\"numbers\\\">0.2</span><span class=\\\"structural\\\"> }</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"false: 0.14967110695957225\\ntrue: 0.15754264158846806\\\"><span class=\\\"structural\\\">{ </span><span class=\\\"structural\\\">false: </span><span class=\\\"numbers\\\">0.1</span><span class=\\\"structural\\\">, </span><span class=\\\"structural\\\">true: </span><span class=\\\"numbers\\\">0.2</span><span class=\\\"structural\\\"> }</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"false: 0.14760448860982958\\ntrue: 0.14203604842767492\\\"><span class=\\\"structural\\\">{ </span><span class=\\\"structural\\\">false: </span><span class=\\\"numbers\\\">0.1</span><span class=\\\"structural\\\">, </span><span class=\\\"structural\\\">true: </span><span class=\\\"numbers\\\">0.1</span><span class=\\\"structural\\\"> }</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"false: 0.17841521012935002\\ntrue: 0.08611590893452556\\\"><span class=\\\"structural\\\">{ </span><span class=\\\"structural\\\">false: </span><span class=\\\"numbers\\\">0.2</span><span class=\\\"structural\\\">, </span><span class=\\\"structural\\\">true: </span><span class=\\\"numbers\\\">0.1</span><span class=\\\"structural\\\"> }</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"false: 0.1562292007202838\\\"><span class=\\\"structural\\\">{ </span><span class=\\\"structural\\\">false: </span><span class=\\\"numbers\\\">0.2</span><span class=\\\"structural\\\"> }</span></span>\"] }, \n",
       "], id: 553648133, rootId: 553648133, totalRows: 6 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648133) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "val smokersData = \n",
    "    data.groupBy { size }\n",
    "        .pivot { smoker }\n",
    "        .mean { tip_pct }\n",
    "        .sortBy { size }\n",
    "smokersData"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "fe722fb3-143f-4730-849d-f6f9926365df",
   "metadata": {},
   "source": [
    "In order to easily process data for plotting, we rearrange data as follows"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "id": "673ad734-dd5f-4a1f-aebb-46fe14c71562",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "application/kotlindataframe+json": "{\"nrow\":12,\"ncol\":3,\"columns\":[\"size\",\"smoker\",\"tip_pct\"],\"kotlin_dataframe\":[{\"size\":1,\"smoker\":\"false\",\"tip_pct\":0.15982946957504365},{\"size\":1,\"smoker\":\"true\",\"tip_pct\":0.27475456139951254},{\"size\":2,\"smoker\":\"false\",\"tip_pct\":0.16499567702041978},{\"size\":2,\"smoker\":\"true\",\"tip_pct\":0.16670580270902313},{\"size\":3,\"smoker\":\"false\",\"tip_pct\":0.14967110695957225},{\"size\":3,\"smoker\":\"true\",\"tip_pct\":0.15754264158846806},{\"size\":4,\"smoker\":\"false\",\"tip_pct\":0.14760448860982958},{\"size\":4,\"smoker\":\"true\",\"tip_pct\":0.14203604842767492},{\"size\":5,\"smoker\":\"false\",\"tip_pct\":0.17841521012935002},{\"size\":5,\"smoker\":\"true\",\"tip_pct\":0.08611590893452556},{\"size\":6,\"smoker\":\"false\",\"tip_pct\":0.1562292007202838},{\"size\":6,\"smoker\":\"true\",\"tip_pct\":null}]}",
      "text/html": [
       "        <html>\n",
       "        <head>\n",
       "            <style type=\"text/css\">\n",
       "                :root {\n",
       "    --background: #fff;\n",
       "    --background-odd: #f5f5f5;\n",
       "    --background-hover: #d9edfd;\n",
       "    --header-text-color: #474747;\n",
       "    --text-color: #848484;\n",
       "    --text-color-dark: #000;\n",
       "    --text-color-medium: #737373;\n",
       "    --text-color-pale: #b3b3b3;\n",
       "    --inner-border-color: #aaa;\n",
       "    --bold-border-color: #000;\n",
       "    --link-color: #296eaa;\n",
       "    --link-color-pale: #296eaa;\n",
       "    --link-hover: #1a466c;\n",
       "}\n",
       "\n",
       ":root[theme=\"dark\"], :root [data-jp-theme-light=\"false\"], .dataframe_dark{\n",
       "    --background: #303030;\n",
       "    --background-odd: #3c3c3c;\n",
       "    --background-hover: #464646;\n",
       "    --header-text-color: #dddddd;\n",
       "    --text-color: #b3b3b3;\n",
       "    --text-color-dark: #dddddd;\n",
       "    --text-color-medium: #b2b2b2;\n",
       "    --text-color-pale: #737373;\n",
       "    --inner-border-color: #707070;\n",
       "    --bold-border-color: #777777;\n",
       "    --link-color: #008dc0;\n",
       "    --link-color-pale: #97e1fb;\n",
       "    --link-hover: #00688e;\n",
       "}\n",
       "\n",
       "p.dataframe_description {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe {\n",
       "    font-family: \"Helvetica Neue\", Helvetica, Arial, sans-serif;\n",
       "    font-size: 12px;\n",
       "    background-color: var(--background);\n",
       "    color: var(--text-color-dark);\n",
       "    border: none;\n",
       "    border-collapse: collapse;\n",
       "}\n",
       "\n",
       "table.dataframe th, td {\n",
       "    padding: 6px;\n",
       "    border: 1px solid transparent;\n",
       "    text-align: left;\n",
       "}\n",
       "\n",
       "table.dataframe th {\n",
       "    background-color: var(--background);\n",
       "    color: var(--header-text-color);\n",
       "}\n",
       "\n",
       "table.dataframe td {\n",
       "    vertical-align: top;\n",
       "}\n",
       "\n",
       "table.dataframe th.bottomBorder {\n",
       "    border-bottom-color: var(--bold-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(odd) {\n",
       "    background: var(--background-odd);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:nth-child(even) {\n",
       "    background: var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe tbody > tr:hover {\n",
       "    background: var(--background-hover);\n",
       "}\n",
       "\n",
       "table.dataframe a {\n",
       "    cursor: pointer;\n",
       "    color: var(--link-color);\n",
       "    text-decoration: none;\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover > td a {\n",
       "    color: var(--link-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe a:hover {\n",
       "    color: var(--link-hover);\n",
       "    text-decoration: underline;\n",
       "}\n",
       "\n",
       "table.dataframe img {\n",
       "    max-width: fit-content;\n",
       "}\n",
       "\n",
       "table.dataframe th.complex {\n",
       "    background-color: var(--background);\n",
       "    border: 1px solid var(--background);\n",
       "}\n",
       "\n",
       "table.dataframe .leftBorder {\n",
       "    border-left-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightBorder {\n",
       "    border-right-color: var(--inner-border-color);\n",
       "}\n",
       "\n",
       "table.dataframe .rightAlign {\n",
       "    text-align: right;\n",
       "}\n",
       "\n",
       "table.dataframe .expanderSvg {\n",
       "    width: 8px;\n",
       "    height: 8px;\n",
       "    margin-right: 3px;\n",
       "}\n",
       "\n",
       "table.dataframe .expander {\n",
       "    display: flex;\n",
       "    align-items: center;\n",
       "}\n",
       "\n",
       "/* formatting */\n",
       "\n",
       "table.dataframe .null {\n",
       "    color: var(--text-color-pale);\n",
       "}\n",
       "\n",
       "table.dataframe .structural {\n",
       "    color: var(--text-color-medium);\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .dataFrameCaption {\n",
       "    font-weight: bold;\n",
       "}\n",
       "\n",
       "table.dataframe .numbers {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe td:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "table.dataframe tr:hover .formatted .structural, .null {\n",
       "    color: var(--text-color-dark);\n",
       "}\n",
       "\n",
       "\n",
       "\n",
       "\n",
       "            </style>\n",
       "        </head>\n",
       "        <body>\n",
       "            \n",
       "<table class=\"dataframe\" id=\"df_553648134\"></table>\n",
       "\n",
       "<p class=\"dataframe_description\">DataFrame: rowsCount = 12, columnsCount = 3</p>\n",
       "        </body>\n",
       "        <script>\n",
       "            \n",
       "/*<!--*/\n",
       "call_DataFrame(function() { DataFrame.addTable({ cols: [{ name: \"<span title=\\\"size: Int\\\">size</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">1</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">2</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">3</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">4</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">5</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">5</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">6</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">6</span></span>\"] }, \n",
       "{ name: \"<span title=\\\"smoker: String\\\">smoker</span>\", children: [], rightAlign: false, values: [\"false\",\"true\",\"false\",\"true\",\"false\",\"true\",\"false\",\"true\",\"false\",\"true\",\"false\",\"true\"] }, \n",
       "{ name: \"<span title=\\\"tip_pct: Double?\\\">tip_pct</span>\", children: [], rightAlign: true, values: [\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.159829</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.274755</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.164996</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.166706</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.149671</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.157543</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.147604</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.142036</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.178415</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.086116</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"numbers\\\">0.156229</span></span>\",\"<span class=\\\"formatted\\\" title=\\\"\\\"><span class=\\\"null\\\">null</span></span>\"] }, \n",
       "], id: 553648134, rootId: 553648134, totalRows: 12 } ) });\n",
       "/*-->*/\n",
       "\n",
       "call_DataFrame(function() { DataFrame.renderTable(553648134) });\n",
       "\n",
       "\n",
       "        </script>\n",
       "        </html>"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "val data = smokersData.flatten().gather(\"false\", \"true\").into(\"smoker\", \"tip_pct\")\n",
    "data"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "38e71527-7e0f-428f-8822-e1cac93f4e36",
   "metadata": {
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "   <div id=\"TZxgRY\"></div>\n",
       "   <script type=\"text/javascript\" data-lets-plot-script=\"plot\">\n",
       "       (function() {\n",
       "           var plotSpec={\n",
       "\"ggtitle\":{\n",
       "\"text\":\"Smoker analysis with Tip Percentage and Goup Size\"\n",
       "},\n",
       "\"mapping\":{\n",
       "\"x\":\"size\",\n",
       "\"y\":\"tip_pct\"\n",
       "},\n",
       "\"data\":{\n",
       "\"smoker\":[\"false\",\"true\",\"false\",\"true\",\"false\",\"true\",\"false\",\"true\",\"false\",\"true\",\"false\",\"true\"],\n",
       "\"size\":[1.0,1.0,2.0,2.0,3.0,3.0,4.0,4.0,5.0,5.0,6.0,6.0],\n",
       "\"tip_pct\":[0.15982946957504365,0.27475456139951254,0.16499567702041978,0.16670580270902313,0.14967110695957225,0.15754264158846806,0.14760448860982958,0.14203604842767492,0.17841521012935002,0.08611590893452556,0.1562292007202838,null]\n",
       "},\n",
       "\"kind\":\"plot\",\n",
       "\"scales\":[{\n",
       "\"aesthetic\":\"y\",\n",
       "\"name\":\"Tip Percentage\"\n",
       "},{\n",
       "\"aesthetic\":\"y\",\n",
       "\"format\":\".1%\"\n",
       "},{\n",
       "\"aesthetic\":\"x\",\n",
       "\"name\":\"Group Size\"\n",
       "}],\n",
       "\"layers\":[{\n",
       "\"mapping\":{\n",
       "\"fill\":\"smoker\"\n",
       "},\n",
       "\"stat\":\"identity\",\n",
       "\"alpha\":0.6,\n",
       "\"position\":{\n",
       "\"name\":\"dodge\",\n",
       "\"width\":0.3\n",
       "},\n",
       "\"geom\":\"bar\",\n",
       "\"tooltips\":{\n",
       "\"variables\":[\"smoker\",\"tip_pct\"],\n",
       "\"formats\":[{\n",
       "\"field\":\"tip_pct\",\n",
       "\"format\":\".1%\"\n",
       "}]\n",
       "},\n",
       "\"data\":{\n",
       "}\n",
       "}]\n",
       "};\n",
       "           var plotContainer = document.getElementById(\"TZxgRY\");\n",
       "           window.letsPlotCall(function() {{\n",
       "               LetsPlot.buildPlotFromProcessedSpecs(plotSpec, -1, -1, plotContainer);\n",
       "           }});\n",
       "       })();    \n",
       "   </script>"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "ggplot(data.toMap()) { x = \"size\" ; y=\"tip_pct\" } +\n",
    "    geomBar(\n",
    "        stat = Stat.identity,\n",
    "        position=positionDodge(0.3),\n",
    "        alpha=0.6,\n",
    "        tooltips = layerTooltips(\"smoker\", \"tip_pct\")\n",
    "            .format(\"tip_pct\", \".1%\")\n",
    "    ) { fill=\"smoker\" } +\n",
    "    ylab(\"Tip Percentage\") +\n",
    "    scaleYContinuous(format=\".1%\") +\n",
    "    xlab(\"Group Size\") +\n",
    "    ggtitle(\"Smoker analysis with Tip Percentage and Goup Size\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "f097d9e0-4ac5-46fb-a90b-a9126e8990fd",
   "metadata": {},
   "source": [
    "We can see that smoker's tip percentage is generally lower that non smoker's. Even on Friday, the day with most smokers, the tips of non-smokers people are higher."
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Kotlin",
   "language": "kotlin",
   "name": "kotlin"
  },
  "language_info": {
   "codemirror_mode": "text/x-kotlin",
   "file_extension": ".kt",
   "mimetype": "text/x-kotlin",
   "name": "kotlin",
   "nbconvert_exporter": "",
   "pygments_lexer": "kotlin",
   "version": "1.8.20"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
